from PyQt5.QtWidgets import QVBoxLayout, QWidget, QLabel, QSpacerItem, QSizePolicy

from rare.utils.extra_widgets import SideTabWidget
from .egl_sync_group import EGLSyncGroup
from .import_group import ImportGroup


class ImportSyncTabs(SideTabWidget):
    def __init__(self, parent=None):
        super(ImportSyncTabs, self).__init__(show_back=True, parent=parent)
        self.import_widget = ImportSyncWidget(
            ImportGroup(self),
            self.tr("To import games from Epic Games Store, please enable EGL Sync."),
            self,
        )
        self.addTab(self.import_widget, self.tr("Import Games"))

        self.egl_sync_widget = ImportSyncWidget(
            EGLSyncGroup(self),
            self.tr("To import EGL games from directories, please use Import Game."),
            self,
        )
        self.addTab(self.egl_sync_widget, self.tr("Sync with EGL"))

        self.tabBar().setCurrentIndex(1)

    def show_import(self):
        self.setCurrentIndex(1)

    def show_egl_sync(self):
        self.setCurrentIndex(2)


class ImportSyncWidget(QWidget):
    def __init__(self, widget: QWidget, info: str, parent=None):
        super(ImportSyncWidget, self).__init__(parent=parent)
        self.info = QLabel(f"<h4>{info}</h4>")

        layout = QVBoxLayout()
        layout.addWidget(widget)
        layout.addWidget(self.info)
        layout.addItem(
            QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)
        )
        self.setLayout(layout)



import os
from logging import getLogger

from PyQt5.QtCore import Qt, QSettings, QTimer, QSize, pyqtSignal, pyqtSlot
from PyQt5.QtGui import QCloseEvent, QCursor
from PyQt5.QtWidgets import QMainWindow, QApplication, QStatusBar, QScrollArea, QScroller, QComboBox, QMessageBox

from rare.components.tabs import TabWidget
from rare.components.tray_icon import TrayIcon
from rare.shared import LegendaryCoreSingleton, GlobalSignalsSingleton, ArgumentsSingleton
from rare.utils.paths import lock_file

logger = getLogger("MainWindow")


class MainWindow(QMainWindow):
    # int: exit code
    exit_app: pyqtSignal = pyqtSignal(int)

    def __init__(self, parent=None):
        super(MainWindow, self).__init__(parent=parent)
        self.setAttribute(Qt.WA_DeleteOnClose, True)
        self.core = LegendaryCoreSingleton()
        self.signals = GlobalSignalsSingleton()
        self.args = ArgumentsSingleton()

        self.settings = QSettings()

        self.setWindowTitle("Rare - GUI for legendary")
        self.tab_widget = TabWidget(self)
        self.setCentralWidget(self.tab_widget)

        self.status_bar = QStatusBar()
        self.setStatusBar(self.status_bar)

        width, height = 1280, 720
        if self.settings.value("save_size", False, bool):
            width, height = self.settings.value("window_size", (width, height), tuple)

        self.resize(width, height)

        if not self.args.offline:
            try:
                from rare.utils.rpc import DiscordRPC
                self.rpc = DiscordRPC()
            except ModuleNotFoundError:
                logger.warning("Discord RPC module not found")

        self.timer = QTimer()
        self.timer.timeout.connect(self.timer_finished)
        self.timer.start(1000)

        self.signals.exit_app.connect(self.on_exit_app)
        self.exit_code = 0
        self.accept_close = False

        self.tray_icon: TrayIcon = TrayIcon(self)
        self.tray_icon.exit_action.triggered.connect(self.on_exit_app)
        self.tray_icon.start_rare.triggered.connect(self.show)
        self.tray_icon.activated.connect(
            lambda r: self.toggle()
            if r == self.tray_icon.DoubleClick
            else None
        )

        self.signals.send_notification.connect(
            lambda title: self.tray_icon.showMessage(
                self.tr("Download finished"),
                self.tr("Download finished. {} is playable now").format(title),
                self.tray_icon.Information,
                4000,
            )
            if self.settings.value("notification", True, bool)
            else None
        )

        self.window_launched = False

        # enable kinetic scrolling
        for scroll_area in self.findChildren(QScrollArea):
            if not scroll_area.property("no_kinetic_scroll"):
                QScroller.grabGesture(scroll_area.viewport(), QScroller.LeftMouseButtonGesture)

            # fix scrolling
            for combo_box in scroll_area.findChildren(QComboBox):
                combo_box.wheelEvent = lambda e: e.ignore()

    def center_window(self):
        # get the margins of the decorated window
        margins = self.windowHandle().frameMargins()
        # get the screen the cursor is on
        current_screen = QApplication.screenAt(QCursor.pos())
        if not current_screen:
            current_screen = QApplication.primaryScreen()
        # get the available screen geometry (excludes panels/docks)
        screen_rect = current_screen.availableGeometry()
        decor_width = margins.left() + margins.right()
        decor_height = margins.top() + margins.bottom()
        window_size = QSize(self.width(), self.height()).boundedTo(
            screen_rect.size() - QSize(decor_width, decor_height)
        )

        self.resize(window_size)
        self.move(
            screen_rect.center()
            - self.rect().adjusted(0, 0, decor_width, decor_height).center()
        )

    def show(self) -> None:
        super(MainWindow, self).show()
        if not self.window_launched:
            self.center_window()
        self.window_launched = True

    def hide(self) -> None:
        if self.settings.value("save_size", False, bool):
            size = self.size().width(), self.size().height()
            self.settings.setValue("window_size", size)
        super(MainWindow, self).hide()

    def toggle(self):
        if self.isHidden():
            self.show()
        else:
            self.hide()

    def timer_finished(self):
        file_path = lock_file()
        if os.path.exists(file_path):
            file = open(file_path, "r")
            action = file.read()
            file.close()
            if action.startswith("show"):
                self.show()
            os.remove(file_path)
        self.timer.start(1000)

    @pyqtSlot()
    @pyqtSlot(int)
    def on_exit_app(self, exit_code=0) -> None:
        # FIXME: Fix this with the download tab redesign
        if not self.args.offline and self.tab_widget.downloadTab.is_download_active:
            question = QMessageBox.question(
                self,
                self.tr("Close"),
                self.tr(
                    "There is a download active. Do you really want to exit app?"
                ),
                QMessageBox.Yes,
                QMessageBox.No,
            )
            if question == QMessageBox.No:
                return
            else:
                # clear queue
                self.tab_widget.downloadTab.queue_widget.update_queue([])
                self.tab_widget.downloadTab.stop_download()
        # FIXME: End of FIXME
        self.exit_code = exit_code
        self.close()

    def close(self) -> bool:
        self.accept_close = True
        return super(MainWindow, self).close()

    def closeEvent(self, e: QCloseEvent):
        if not self.accept_close:
            if self.settings.value("sys_tray", True, bool):
                self.hide()
                e.ignore()
                return
        self.timer.stop()
        self.tray_icon.deleteLater()
        self.hide()
        self.exit_app.emit(self.exit_code)
        super(MainWindow, self).closeEvent(e)
        e.accept()



import platform
import re
from logging import getLogger
from typing import Tuple

from PyQt5.QtCore import Qt, QRunnable, QObject, pyqtSignal, QThreadPool, QSettings
from PyQt5.QtWidgets import QSizePolicy, QWidget, QFileDialog, QMessageBox

from rare.components.tabs.settings.widgets.eos import EosWidget
from rare.components.tabs.settings.widgets.ubisoft_activation import UbiActivationHelper
from rare.shared import LegendaryCoreSingleton
from rare.ui.components.tabs.settings.legendary import Ui_LegendarySettings
from rare.utils.extra_widgets import PathEdit, IndicatorLineEdit
from rare.utils.misc import get_size

logger = getLogger("LegendarySettings")


class RefreshGameMetaSignals(QObject):
    finished = pyqtSignal()

    def __init__(self):
        super(RefreshGameMetaSignals, self).__init__()


class RefreshGameMetaWorker(QRunnable):
    def __init__(self):
        super(RefreshGameMetaWorker, self).__init__()
        self.signals = RefreshGameMetaSignals()
        self.setAutoDelete(True)
        self.core = LegendaryCoreSingleton()

    def run(self) -> None:
        self.core.get_game_and_dlc_list(True, force_refresh=True)
        self.signals.finished.emit()


class LegendarySettings(QWidget, Ui_LegendarySettings):
    def __init__(self, parent=None):
        super(LegendarySettings, self).__init__(parent=parent)
        self.setupUi(self)
        self.settings = QSettings()

        self.core = LegendaryCoreSingleton()

        # Default installation directory
        self.install_dir = PathEdit(
            self.core.get_default_install_dir(),
            file_type=QFileDialog.DirectoryOnly,
            save_func=self.path_save,
        )
        self.install_dir_layout.addWidget(self.install_dir)

        # Max Workers
        max_workers = self.core.lgd.config["Legendary"].getint(
            "max_workers", fallback=0
        )
        self.max_worker_spin.setValue(max_workers)
        self.max_worker_spin.valueChanged.connect(self.max_worker_save)
        # Max memory
        max_memory = self.core.lgd.config["Legendary"].getint("max_memory", fallback=0)
        self.max_memory_spin.setValue(max_memory)
        self.max_memory_spin.valueChanged.connect(self.max_memory_save)
        # Preferred CDN
        preferred_cdn = self.core.lgd.config["Legendary"].get(
            "preferred_cdn", fallback=""
        )
        self.preferred_cdn_line.setText(preferred_cdn)
        self.preferred_cdn_line.textChanged.connect(self.preferred_cdn_save)
        # Disable HTTPS
        disable_https = self.core.lgd.config["Legendary"].getboolean(
            "disable_https", fallback=False
        )
        self.disable_https_check.setChecked(disable_https)
        self.disable_https_check.stateChanged.connect(self.disable_https_save)

        # Cleanup
        self.clean_button.clicked.connect(lambda: self.cleanup(False))
        self.clean_keep_manifests_button.clicked.connect(lambda: self.cleanup(True))

        self.locale_edit = IndicatorLineEdit(
            f"{self.core.language_code}-{self.core.country_code}",
            edit_func=self.locale_edit_cb,
            save_func=self.locale_save_cb,
            horiz_policy=QSizePolicy.Minimum,
            parent=self,
        )
        self.locale_layout.addWidget(self.locale_edit)

        self.ubi_helper = UbiActivationHelper(self.ubisoft_gb)
        self.eos_widget = EosWidget()
        self.left_layout.insertWidget(3, self.eos_widget, alignment=Qt.AlignTop)

        self.win32_cb.setChecked(self.settings.value("win32_meta", False, bool))
        self.win32_cb.stateChanged.connect(lambda: self.settings.setValue("win32_meta", self.win32_cb.isChecked()))

        self.mac_cb.setChecked(self.settings.value("mac_meta", platform.system() == "Darwin", bool))
        self.mac_cb.stateChanged.connect(lambda: self.settings.setValue("mac_meta", self.mac_cb.isChecked()))

        self.refresh_game_meta_btn.clicked.connect(self.refresh_game_meta)

    def refresh_game_meta(self):
        self.refresh_game_meta_btn.setDisabled(True)
        self.refresh_game_meta_btn.setText(self.tr("Loading"))
        worker = RefreshGameMetaWorker()
        worker.signals.finished.connect(lambda: self.refresh_game_meta_btn.setDisabled(False))
        worker.signals.finished.connect(lambda: self.refresh_game_meta_btn.setText(self.tr("Refresh game meta")))
        QThreadPool.globalInstance().start(worker)

    @staticmethod
    def locale_edit_cb(text: str) -> Tuple[bool, str, str]:
        if text:
            if re.match("^[a-zA-Z]{2,3}[-_][a-zA-Z]{2,3}$", text):
                language, country = text.replace("_", "-").split("-")
                text = "-".join([language.lower(), country.upper()])
            if bool(re.match("^[a-z]{2,3}-[A-Z]{2,3}$", text)):
                return True, text, ""
            else:
                return False, text, IndicatorLineEdit.reasons.wrong_format

        else:
            return True, text, ""

    def locale_save_cb(self, text: str):
        if text:
            self.core.egs.language_code, self.core.egs.country_code = text.split("-")
            self.core.lgd.config.set("Legendary", "locale", text)
        else:
            if self.core.lgd.config.has_option("Legendary", "locale"):
                self.core.lgd.config.remove_option("Legendary", "locale")
        self.core.lgd.save_config()

    def path_save(self, text: str):
        self.core.lgd.config["Legendary"]["install_dir"] = text
        if not text and "install_dir" in self.core.lgd.config["Legendary"].keys():
            self.core.lgd.config["Legendary"].pop("install_dir")
        else:
            logger.debug(f"Set config install_dir to {text}")
        self.core.lgd.save_config()

    def max_worker_save(self, workers: str):
        if workers := int(workers):
            self.core.lgd.config.set("Legendary", "max_workers", str(workers))
        else:
            self.core.lgd.config.remove_option("Legendary", "max_workers")
        self.core.lgd.save_config()

    def max_memory_save(self, memory: str):
        if memory := int(memory):
            self.core.lgd.config.set("Legendary", "max_memory", str(memory))
        else:
            self.core.lgd.config.remove_option("Legendary", "max_memory")
        self.core.lgd.save_config()

    def preferred_cdn_save(self, cdn: str):
        if cdn:
            self.core.lgd.config.set("Legendary", "preferred_cdn", cdn.strip())
        else:
            self.core.lgd.config.remove_option("Legendary", "preferred_cdn")
        self.core.lgd.save_config()

    def disable_https_save(self, checked: int):
        self.core.lgd.config.set(
            "Legendary", "disable_https", str(bool(checked)).lower()
        )
        self.core.lgd.save_config()

    def cleanup(self, keep_manifests: bool):
        before = self.core.lgd.get_dir_size()
        logger.debug("Removing app metadata...")
        app_names = set(g.app_name for g in self.core.get_assets(update_assets=False))
        self.core.lgd.clean_metadata(app_names)

        if not keep_manifests:
            logger.debug("Removing manifests...")
            installed = [
                (ig.app_name, ig.version, ig.platform) for ig in self.core.get_installed_list()
            ]
            installed.extend(
                (ig.app_name, ig.version, ig.platform) for ig in self.core.get_installed_dlc_list()
            )
            self.core.lgd.clean_manifests(installed)

        logger.debug("Removing tmp data")
        self.core.lgd.clean_tmp_data()

        after = self.core.lgd.get_dir_size()
        logger.info(
            f"Cleanup complete! Removed {(before - after) / 1024 / 1024:.02f} MiB."
        )
        if (before - after) > 0:
            QMessageBox.information(
                self,
                "Cleanup",
                self.tr("Cleanup complete! Successfully removed {}").format(
                    get_size(before - after)
                ),
            )
        else:
            QMessageBox.information(self, "Cleanup", "Nothing to clean")



import os
import platform
from logging import getLogger

from PyQt5.QtCore import pyqtSignal, QObject, QRunnable, QStandardPaths
from legendary.core import LegendaryCore

from rare.lgndr.api_arguments import LgndrVerifyGameArgs, LgndrUninstallGameArgs
from rare.lgndr.api_monkeys import LgndrIndirectStatus
from rare.lgndr.cli import LegendaryCLI
from rare.shared import LegendaryCoreSingleton, ArgumentsSingleton
from rare.utils import config_helper

logger = getLogger("Legendary Utils")


def uninstall_game(core: LegendaryCore, app_name: str, keep_files=False, keep_config=False):
    igame = core.get_installed_game(app_name)

    # remove shortcuts link
    desktop = QStandardPaths.writableLocation(QStandardPaths.DesktopLocation)
    applications = QStandardPaths.writableLocation(QStandardPaths.ApplicationsLocation)
    if platform.system() == "Linux":
        desktop_shortcut = os.path.join(desktop, f"{igame.title}.desktop")
        if os.path.exists(desktop_shortcut):
            os.remove(desktop_shortcut)

        applications_shortcut = os.path.join(applications, f"{igame.title}.desktop")
        if os.path.exists(applications_shortcut):
            os.remove(applications_shortcut)

    elif platform.system() == "Windows":
        game_title = igame.title.split(":")[0]
        desktop_shortcut = os.path.join(desktop, f"{game_title}.lnk")
        if os.path.exists(desktop_shortcut):
            os.remove(desktop_shortcut)

        start_menu_shortcut = os.path.join(applications, "..", f"{game_title}.lnk")
        if os.path.exists(start_menu_shortcut):
            os.remove(start_menu_shortcut)

    status = LgndrIndirectStatus()
    LegendaryCLI(core).uninstall_game(
        LgndrUninstallGameArgs(
            app_name=app_name,
            keep_files=keep_files,
            indirect_status=status,
            yes=True,
        )
    )
    if not keep_config:
        logger.info("Removing sections in config file")
        config_helper.remove_section(app_name)
        config_helper.remove_section(f"{app_name}.env")

        config_helper.save_config()

    return status.success, status.message


def update_manifest(app_name: str, core: LegendaryCore):
    game = core.get_game(app_name)
    logger.info(f"Reloading game manifest of {game.app_title}")
    new_manifest_data, base_urls = core.get_cdn_manifest(game)
    # overwrite base urls in metadata with current ones to avoid using old/dead CDNs
    game.base_urls = base_urls
    # save base urls to game metadata
    core.lgd.set_game_meta(game.app_name, game)

    new_manifest = core.load_manifest(new_manifest_data)
    logger.debug(f"Base urls: {base_urls}")
    # save manifest with version name as well for testing/downgrading/etc.
    core.lgd.save_manifest(game.app_name, new_manifest_data, version=new_manifest.meta.build_version)


class VerifyWorker(QRunnable):
    class Signals(QObject):
        status = pyqtSignal(str, int, int, float, float)
        result = pyqtSignal(str, bool, int, int)
        error = pyqtSignal(str, str)

    num: int = 0
    total: int = 1  # set default to 1 to avoid DivisionByZero before it is initialized

    def __init__(self, app_name):
        super(VerifyWorker, self).__init__()
        self.signals = VerifyWorker.Signals()
        self.setAutoDelete(True)
        self.core = LegendaryCoreSingleton()
        self.args = ArgumentsSingleton()
        self.app_name = app_name

    def status_callback(self, num: int, total: int, percentage: float, speed: float):
        self.signals.status.emit(self.app_name, num, total, percentage, speed)

    def run(self):
        cli = LegendaryCLI(self.core)
        status = LgndrIndirectStatus()
        args = LgndrVerifyGameArgs(
            app_name=self.app_name, indirect_status=status, verify_stdout=self.status_callback
        )

        # lk: first pass, verify with the current manifest
        repair_mode = False
        result = cli.verify_game(
            args, print_command=False, repair_mode=repair_mode, repair_online=not self.args.offline
        )
        if result is None:
            # lk: second pass with downloading the latest manifest
            # lk: this happens if the manifest was not found and repair_mode was not requested
            # lk: we already have checked if the directory exists before starting the worker
            try:
                # lk: this try-except block handles the exception caused by a missing manifest
                # lk: and is raised only in the case we are offline
                repair_mode = True
                result = cli.verify_game(
                    args, print_command=False, repair_mode=repair_mode, repair_online=not self.args.offline
                )
                if result is None:
                    raise ValueError
            except ValueError:
                self.signals.error.emit(self.app_name, status.message)
                return

        success = result is not None and not any(result)
        if success:
            # lk: if verification was successful we delete the repair file and run the clean procedure
            # lk: this could probably be cut down to what is relevant for this use-case and skip the `cli` call
            igame = self.core.get_installed_game(self.app_name)
            game = self.core.get_game(self.app_name, platform=igame.platform)
            repair_file = os.path.join(self.core.lgd.get_tmp_path(), f"{self.app_name}.repair")
            cli.install_game_cleanup(game=game, igame=igame, repair_mode=True, repair_file=repair_file)

        self.signals.result.emit(self.app_name, success, *result)



import json
import os
from dataclasses import dataclass
from enum import IntEnum
from logging import getLogger
from pathlib import Path
from typing import List, Tuple, Optional

from PyQt5.QtCore import Qt, QModelIndex, pyqtSignal, QRunnable, QObject, QThreadPool, pyqtSlot
from PyQt5.QtGui import QStandardItemModel
from PyQt5.QtWidgets import QFileDialog, QGroupBox, QCompleter, QTreeView, QHeaderView, QMessageBox

from rare.lgndr.api_arguments import LgndrImportGameArgs
from rare.lgndr.api_monkeys import LgndrIndirectStatus
from rare.lgndr.cli import LegendaryCLI
from rare.shared import LegendaryCoreSingleton, GlobalSignalsSingleton, ApiResultsSingleton
from rare.ui.components.tabs.games.import_sync.import_group import Ui_ImportGroup
from rare.utils.extra_widgets import IndicatorLineEdit, PathEdit
from rare.widgets.elide_label import ElideLabel

logger = getLogger("Import")


def find_app_name(path: str, core) -> Optional[str]:
    if os.path.exists(os.path.join(path, ".egstore")):
        for i in os.listdir(os.path.join(path, ".egstore")):
            if i.endswith(".mancpn"):
                with open(os.path.join(path, ".egstore", i)) as file:
                    app_name = json.load(file).get("AppName")
                return app_name
    elif app_name := LegendaryCLI(core).resolve_aliases(os.path.basename(os.path.normpath(path))):
        # return None if game does not exist (Workaround for overlay)
        if not core.get_game(app_name):
            return None
        return app_name
    else:
        logger.warning(f"Could not find AppName for {path}")
    return None


class ImportResult(IntEnum):
    ERROR = 0
    FAILED = 1
    SUCCESS = 2


@dataclass
class ImportedGame:
    result: ImportResult
    path: Optional[str] = None
    app_name: Optional[str] = None
    app_title: Optional[str] = None
    message: Optional[str] = None


class ImportWorker(QRunnable):
    class Signals(QObject):
        finished = pyqtSignal(list)
        progress = pyqtSignal(int)

    def __init__(self, path: str, app_name: str = None, import_folder: bool = False, import_dlcs: bool = False):
        super(ImportWorker, self).__init__()
        self.signals = self.Signals()
        self.core = LegendaryCoreSingleton()

        self.path = Path(path)
        self.app_name = app_name
        self.import_folder = import_folder
        self.import_dlcs = import_dlcs

    def run(self) -> None:
        result_list: List = []
        if self.import_folder:
            folders = [i for i in self.path.iterdir() if i.is_dir()]
            number_of_folders = len(folders)
            for i, child in enumerate(folders):
                if not child.is_dir():
                    continue
                result = self.__try_import(child, None)
                result_list.append(result)
                self.signals.progress.emit(int(100 * i // number_of_folders))
        else:
            result = self.__try_import(self.path, self.app_name)
            result_list.append(result)
        self.signals.finished.emit(result_list)

    def __try_import(self, path: Path, app_name: str = None) -> ImportedGame:
        result = ImportedGame(ImportResult.ERROR)
        result.path = str(path)
        if app_name or (app_name := find_app_name(str(path), self.core)):
            result.app_name = app_name
            result.app_title = app_title = self.core.get_game(app_name).app_title
            success, message = self.__import_game(path, app_name, app_title)
            if not success:
                result.result = ImportResult.FAILED
                result.message = message
            else:
                result.result = ImportResult.SUCCESS
        return result

    def __import_game(self, path: Path, app_name: str, app_title: str):
        cli = LegendaryCLI(self.core)
        status = LgndrIndirectStatus()
        args = LgndrImportGameArgs(
            app_path=str(path),
            app_name=app_name,
            indirect_status=status,
            get_boolean_choice=lambda prompt, default=True: self.import_dlcs
        )
        cli.import_game(args)
        return status.success, status.message


class AppNameCompleter(QCompleter):
    activated = pyqtSignal(str)

    def __init__(self, app_names: List, parent=None):
        super(AppNameCompleter, self).__init__(parent)
        # pylint: disable=E1136
        super(AppNameCompleter, self).activated[QModelIndex].connect(self.__activated_idx)

        model = QStandardItemModel(len(app_names), 2)
        for idx, game in enumerate(app_names):
            app_name, app_title = game
            model.setData(model.index(idx, 0), app_title)
            model.setData(model.index(idx, 1), app_name)
        self.setModel(model)

        treeview = QTreeView()
        self.setPopup(treeview)
        treeview.setRootIsDecorated(False)
        treeview.header().hide()
        treeview.header().setSectionResizeMode(0, QHeaderView.Stretch)
        treeview.header().setSectionResizeMode(1, QHeaderView.Stretch)

        # listview = QListView()
        # self.setPopup(listview)
        # # listview.setModelColumn(1)

        self.setFilterMode(Qt.MatchContains)
        self.setCaseSensitivity(Qt.CaseInsensitive)
        # self.setCompletionMode(QCompleter.UnfilteredPopupCompletion)

    def __activated_idx(self, idx):
        # lk: don't even look at this in a funny way, it will die of shame
        # lk: Note to self, the completer and popup models are different.
        # lk: Getting the index from the popup and trying to use it in the completer will return invalid results
        if isinstance(idx, QModelIndex):
            self.activated.emit(self.popup().model().data(self.popup().model().index(idx.row(), 1)))
        # TODO: implement conversion from app_name to app_title (signal loop here)
        # if isinstance(idx_str, str):
        #     self.activated.emit(idx_str)


class ImportGroup(QGroupBox):
    def __init__(self, parent=None):
        super(ImportGroup, self).__init__(parent=parent)
        self.ui = Ui_ImportGroup()
        self.ui.setupUi(self)
        self.core = LegendaryCoreSingleton()
        self.signals = GlobalSignalsSingleton()
        self.api_results = ApiResultsSingleton()

        self.app_name_list = [game.app_name for game in self.api_results.game_list]
        self.install_dir_list = [
            game.metadata.get("customAttributes", {})
            .get("FolderName", {})
            .get("value", game.app_name)
            for game in self.api_results.game_list
            if not game.is_dlc
        ]

        self.path_edit = PathEdit(
            self.core.get_default_install_dir(),
            QFileDialog.DirectoryOnly,
            edit_func=self.path_edit_cb,
            parent=self,
        )
        self.path_edit.textChanged.connect(self.path_changed)
        self.ui.path_edit_layout.addWidget(self.path_edit)

        self.app_name_edit = IndicatorLineEdit(
            placeholder=self.tr("Use in case the app name was not found automatically"),
            completer=AppNameCompleter(
                app_names=[(i.app_name, i.app_title) for i in self.api_results.game_list]
            ),
            edit_func=self.app_name_edit_cb,
            parent=self,
        )
        self.app_name_edit.textChanged.connect(self.app_name_changed)
        self.ui.app_name_layout.addWidget(self.app_name_edit)

        self.ui.import_folder_check.stateChanged.connect(self.import_folder_changed)
        self.ui.import_dlcs_check.setEnabled(False)

        self.ui.import_button.setEnabled(False)
        self.ui.import_button.clicked.connect(
            lambda: self.import_pressed(self.path_edit.text())
        )

        self.info_label = ElideLabel(text="", parent=self)
        self.ui.button_info_layout.addWidget(self.info_label)

        self.threadpool = QThreadPool.globalInstance()

    def path_edit_cb(self, path) -> Tuple[bool, str, str]:
        if os.path.exists(path):
            if os.path.exists(os.path.join(path, ".egstore")):
                return True, path, ""
            elif os.path.basename(path) in self.install_dir_list:
                return True, path, ""
        else:
            return False, path, PathEdit.reasons.dir_not_exist
        return False, path, ""

    def path_changed(self, path):
        self.info_label.setText("")
        self.ui.import_folder_check.setCheckState(Qt.Unchecked)
        if self.path_edit.is_valid:
            self.app_name_edit.setText(find_app_name(path, self.core))
        else:
            self.app_name_edit.setText("")

    def app_name_edit_cb(self, text) -> Tuple[bool, str, str]:
        if not text:
            return False, text, ""
        if text in self.app_name_list:
            return True, text, ""
        else:
            return False, text, IndicatorLineEdit.reasons.game_not_installed

    def app_name_changed(self, app_name: str):
        self.info_label.setText("")
        self.ui.import_dlcs_check.setCheckState(Qt.Unchecked)
        if self.app_name_edit.is_valid:
            self.ui.import_dlcs_check.setEnabled(
                bool(self.core.get_dlc_for_game(app_name))
            )
            self.ui.import_button.setEnabled(True)
        else:
            self.ui.import_dlcs_check.setEnabled(False)
            self.ui.import_button.setEnabled(False)

    def import_folder_changed(self, state):
        self.app_name_edit.setEnabled(not state)
        self.ui.import_dlcs_check.setCheckState(Qt.Unchecked)
        self.ui.import_dlcs_check.setEnabled(
            state
            or (self.app_name_edit.is_valid and bool(self.core.get_dlc_for_game(self.app_name_edit.text())))
        )
        self.ui.import_button.setEnabled(state or (not state and self.app_name_edit.is_valid))

    def import_pressed(self, path=None):
        if not path:
            path = self.path_edit.text()
        worker = ImportWorker(
            path,
            self.app_name_edit.text(),
            self.ui.import_folder_check.isChecked(),
            self.ui.import_dlcs_check.isChecked(),
        )
        worker.signals.finished.connect(self.import_finished)
        worker.signals.progress.connect(self.import_progress)
        self.threadpool.start(worker)
        self.info_label.setText(self.tr("Importing games"))
        self.ui.import_button.setDisabled(True)

    @pyqtSlot(list)
    def import_finished(self, result: List):
        logger.info(f"Import finished: {result}")
        self.info_label.setText("")

        self.signals.update_gamelist.emit([r.app_name for r in result if r.result == ImportResult.SUCCESS])

        for failed in (f for f in result if f.result == ImportResult.FAILED):
            igame = self.core.get_installed_game(failed.app_name)
            if igame and igame.version != self.core.get_asset(igame.app_name, igame.platform, False).build_version:
                # update available
                self.signals.add_download.emit(igame.app_name)
                self.signals.update_download_tab_text.emit()

        if len(result) == 1:
            res = result[0]
            if res.result == ImportResult.SUCCESS:
                self.info_label.setText(
                    self.tr("Success: <b>{}</b> imported").format(res.app_title)
                )
            elif res.result == ImportResult.FAILED:
                self.info_label.setText(
                    self.tr("Failed: <b>{}</b> - {}").format(res.app_title, res.message)
                )
            else:
                self.info_label.setText(
                    self.tr("Error: Could not find AppName for <b>{}</b>").format(res.path)
                )
        else:
            success = [r for r in result if r.result == ImportResult.SUCCESS]
            failure = [r for r in result if r.result == ImportResult.FAILED]
            errored = [r for r in result if r.result == ImportResult.ERROR]
            # pylint: disable=E1101
            messagebox = QMessageBox(
                QMessageBox.Information,
                self.tr("Import summary"),
                self.tr(
                    "Tried to import {} folders.\n\n"
                    "Successfully imported {} games, failed to import {} games and {} errors occurred"
                ).format(len(success) + len(failure) + len(errored), len(success), len(failure), len(errored)),
                buttons=QMessageBox.StandardButton.Close,
                parent=self,
            )
            messagebox.setWindowModality(Qt.NonModal)
            details: List = []
            for res in success:
                details.append(
                    self.tr("Success: {} imported").format(res.app_title)
                )
            for res in failure:
                details.append(
                    self.tr("Failed: {} - {}").format(res.app_title, res.message)
                )
            for res in errored:
                details.append(
                    self.tr("Error: Could not find AppName for {}").format(res.path)
                )
            messagebox.setDetailedText("\n".join(details))
            messagebox.show()

    def import_progress(self, progress: int):
        pass



from dataclasses import dataclass
from typing import Optional, List, Dict


@dataclass
class ApiResults:
    game_list: Optional[List] = None
    dlcs: Optional[Dict] = None
    bit32_games: Optional[List] = None
    mac_games: Optional[List] = None
    no_asset_games: Optional[List] = None
    saves: Optional[List] = None

    def __bool__(self):
        return (
            self.game_list is not None
            and self.dlcs is not None
            and self.bit32_games is not None
            and self.mac_games is not None
            and self.no_asset_games is not None
            and self.saves is not None
        )



import os
import platform
import queue
import time
from dataclasses import dataclass
from enum import IntEnum
from logging import getLogger
from typing import List, Optional, Dict

from PyQt5.QtCore import QThread, pyqtSignal, QProcess
from legendary.core import LegendaryCore

from rare.lgndr.api_monkeys import DLManagerSignals
from rare.lgndr.cli import LegendaryCLI
from rare.lgndr.downloading import UIUpdate
from rare.models.install import InstallQueueItemModel
from rare.shared import GlobalSignalsSingleton, ArgumentsSingleton

logger = getLogger("DownloadThread")


class DownloadThread(QThread):
    @dataclass
    class ReturnStatus:
        class ReturnCode(IntEnum):
            ERROR = 1
            STOPPED = 2
            FINISHED = 3

        app_name: str
        ret_code: ReturnCode = ReturnCode.ERROR
        message: str = ""
        dlcs: Optional[List[Dict]] = None
        sync_saves: bool = False
        tip_url: str = ""
        shortcuts: bool = False

    ret_status = pyqtSignal(ReturnStatus)
    ui_update = pyqtSignal(UIUpdate)

    def __init__(self, core: LegendaryCore, item: InstallQueueItemModel):
        super(DownloadThread, self).__init__()
        self.signals = GlobalSignalsSingleton()
        self.core: LegendaryCore = core
        self.item: InstallQueueItemModel = item
        self.dlm_signals: DLManagerSignals = DLManagerSignals()

    def run(self):
        cli = LegendaryCLI(self.core)
        self.item.download.dlm.logging_queue = cli.logging_queue
        self.item.download.dlm.proc_debug = ArgumentsSingleton().debug
        ret = DownloadThread.ReturnStatus(self.item.download.game.app_name)
        start_t = time.time()
        try:
            self.item.download.dlm.start()
            time.sleep(1)
            while self.item.download.dlm.is_alive():
                try:
                    self.ui_update.emit(self.item.download.dlm.status_queue.get(timeout=1.0))
                except queue.Empty:
                    pass
                if self.dlm_signals.update:
                    try:
                        self.item.download.dlm.signals_queue.put(self.dlm_signals, block=False, timeout=1.0)
                    except queue.Full:
                        pass
                time.sleep(self.item.download.dlm.update_interval / 10)
            self.item.download.dlm.join()
        except Exception as e:
            end_t = time.time()
            logger.error(f"Installation failed after {end_t - start_t:.02f} seconds.")
            logger.warning(f"The following exception occurred while waiting for the downloader to finish: {e!r}.")
            ret.ret_code = ret.ReturnCode.ERROR
            ret.message = f"{e!r}"
            self.ret_status.emit(ret)
            return
        else:
            end_t = time.time()
            if self.dlm_signals.kill is True:
                logger.info(f"Download stopped after {end_t - start_t:.02f} seconds.")
                ret.ret_code = ret.ReturnCode.STOPPED
                self.ret_status.emit(ret)
                return
            logger.info(f"Download finished in {end_t - start_t:.02f} seconds.")

            ret.ret_code = ret.ReturnCode.FINISHED

            if self.item.options.overlay:
                self.signals.overlay_installation_finished.emit()
                self.core.finish_overlay_install(self.item.download.igame)
                self.ret_status.emit(ret)
                return

            if not self.item.options.no_install:
                postinstall = self.core.install_game(self.item.download.igame)
                if postinstall:
                    # LegendaryCLI(self.core)._handle_postinstall(
                    #     postinstall,
                    #     self.item.download.igame,
                    #     False,
                    #     self.item.options.install_prereqs,
                    # )
                    self._handle_postinstall(postinstall, self.item.download.igame)

                dlcs = self.core.get_dlc_for_game(self.item.download.igame.app_name)
                if dlcs and not self.item.options.skip_dlcs:
                    ret.dlcs = []
                    for dlc in dlcs:
                        ret.dlcs.append(
                            {
                                "app_name": dlc.app_name,
                                "app_title": dlc.app_title,
                                "app_version": dlc.app_version(self.item.options.platform),
                            }
                        )

                if (
                    self.item.download.game.supports_cloud_saves
                    or self.item.download.game.supports_mac_cloud_saves
                ) and not self.item.download.game.is_dlc:
                    ret.sync_saves = True

                # show tip again after installation finishes so users hopefully actually see it
                if tip_url := self.core.get_game_tip(self.item.download.igame.app_name):
                    ret.tip_url = tip_url

            LegendaryCLI(self.core).install_game_cleanup(
                self.item.download.game,
                self.item.download.igame,
                self.item.download.repair,
                self.item.download.repair_file,
            )

            if not self.item.options.update and self.item.options.create_shortcut:
                ret.shortcuts = True

        self.ret_status.emit(ret)

    def _handle_postinstall(self, postinstall, igame):
        logger.info("This game lists the following prerequisites to be installed:")
        logger.info(f'- {postinstall["name"]}: {" ".join((postinstall["path"], postinstall["args"]))}')
        if platform.system() == "Windows":
            if self.item.options.install_prereqs:
                logger.info("Launching prerequisite executable..")
                self.core.prereq_installed(igame.app_name)
                req_path, req_exec = os.path.split(postinstall["path"])
                work_dir = os.path.join(igame.install_path, req_path)
                fullpath = os.path.join(work_dir, req_exec)
                proc = QProcess()
                proc.setProcessChannelMode(QProcess.MergedChannels)
                proc.readyReadStandardOutput.connect(
                    lambda: logger.debug(str(proc.readAllStandardOutput().data(), "utf-8", "ignore"))
                )
                proc.setProgram(fullpath)
                proc.setArguments(postinstall.get("args", "").split(" "))
                proc.setWorkingDirectory(work_dir)
                proc.start()
                proc.waitForFinished()  # wait, because it is inside the thread
        else:
            logger.info("Automatic installation not available on Linux.")

    def kill(self):
        self.dlm_signals.kill = True



import os
from logging import getLogger
from pathlib import Path
from typing import Tuple

from PyQt5.QtWidgets import QGroupBox, QFileDialog

from rare.components.tabs.settings import LinuxSettings
from .wrapper import WrapperSettings
from rare.ui.components.tabs.settings.proton import Ui_ProtonSettings
from rare.utils import config_helper
from rare.utils.extra_widgets import PathEdit
from rare.shared import LegendaryCoreSingleton

logger = getLogger("Proton")


def find_proton_combos():
    possible_proton_combos = []
    compatibilitytools_dirs = [
        os.path.expanduser("~/.steam/steam/steamapps/common"),
        "/usr/share/steam/compatibilitytools.d",
        os.path.expanduser("~/.steam/compatibilitytools.d"),
        os.path.expanduser("~/.steam/root/compatibilitytools.d"),
    ]
    for c in compatibilitytools_dirs:
        if os.path.exists(c):
            for i in os.listdir(c):
                proton = os.path.join(c, i, "proton")
                compatibilitytool = os.path.join(c, i, "compatibilitytool.vdf")
                toolmanifest = os.path.join(c, i, "toolmanifest.vdf")
                if os.path.exists(proton) and (
                        os.path.exists(compatibilitytool) or os.path.exists(toolmanifest)
                ):
                    wrapper = f'"{proton}" run'
                    possible_proton_combos.append(wrapper)
    if not possible_proton_combos:
        logger.warning("Unable to find any Proton version")
    return possible_proton_combos


class ProtonSettings(QGroupBox, Ui_ProtonSettings):

    app_name: str
    changeable = True

    def __init__(self, linux_settings: LinuxSettings, wrapper_settings: WrapperSettings):
        super(ProtonSettings, self).__init__()
        self.setupUi(self)
        self._linux_settings = linux_settings
        self._wrapper_settings = wrapper_settings
        self.core = LegendaryCoreSingleton()
        self.possible_proton_combos = find_proton_combos()

        self.proton_combo.addItems(self.possible_proton_combos)
        self.proton_combo.currentIndexChanged.connect(self.change_proton)

        self.proton_prefix = PathEdit(
            file_type=QFileDialog.DirectoryOnly,
            edit_func=self.proton_prefix_edit,
            save_func=self.proton_prefix_save,
            placeholder=self.tr("Please select path for proton prefix")
        )
        self.prefix_layout.addWidget(self.proton_prefix)

    def change_proton(self, i):
        if not self.changeable:
            return
        # First combo box entry: Don't use Proton
        if i == 0:
            self._wrapper_settings.delete_wrapper("proton")
            config_helper.remove_option(self.app_name, "no_wine")
            config_helper.remove_option(f"{self.app_name}.env", "STEAM_COMPAT_DATA_PATH")
            config_helper.remove_option(f"{self.app_name}.env", "STEAM_COMPAT_CLIENT_INSTALL_PATH")

            self.proton_prefix.setEnabled(False)
            # lk: TODO: This has to be fixed properly.
            # lk: It happens because of the widget update. Mask it for now behind disabling the save button

            self._linux_settings.wine_groupbox.setEnabled(True)
        else:
            self.proton_prefix.setEnabled(True)
            self._linux_settings.wine_groupbox.setEnabled(False)
            wrapper = self.possible_proton_combos[i - 1]
            self._wrapper_settings.add_wrapper(wrapper)
            config_helper.add_option(self.app_name, "no_wine", "true")
            config_helper.add_option(
                f"{self.app_name}.env",
                "STEAM_COMPAT_DATA_PATH",
                os.path.expanduser("~/.proton"),
            )
            config_helper.add_option(
                f"{self.app_name}.env",
                "STEAM_COMPAT_CLIENT_INSTALL_PATH",
                str(Path.home().joinpath(".steam", "steam"))
            )

            self.proton_prefix.setText(os.path.expanduser("~/.proton"))

            # Don't use Wine
            self._linux_settings.wine_exec.setText("")
            self._linux_settings.wine_prefix.setText("")

        config_helper.save_config()

    def proton_prefix_edit(self, text: str) -> Tuple[bool, str, str]:
        if not text:
            text = os.path.expanduser("~/.proton")
            return True, text, ""
        parent_dir = os.path.dirname(text)
        return os.path.exists(parent_dir), text, PathEdit.reasons.dir_not_exist

    def proton_prefix_save(self, text: str):
        if not self.changeable:
            return
        config_helper.add_option(
            f"{self.app_name}.env", "STEAM_COMPAT_DATA_PATH", text
        )
        config_helper.save_config()

    def load_settings(self, app_name: str, proton: str):
        self.changeable = False
        self.app_name = app_name
        proton = proton.replace('"', "")
        self.proton_prefix.setEnabled(bool(proton))
        if proton:
            print(proton)
            self.proton_combo.setCurrentText(
                f'"{proton.replace(" run", "")}" run'
            )

        else:
            self.proton_combo.setCurrentIndex(0)

        proton_prefix = self.core.lgd.config.get(
            f"{app_name}.env",
            "STEAM_COMPAT_DATA_PATH",
            fallback=str(Path.home().joinpath(".proton")),
        )
        self.proton_prefix.setText(proton_prefix)
        self.changeable = True



from logging import getLogger

from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QLabel, QHBoxLayout, QVBoxLayout, QPushButton

from legendary.core import LegendaryCore
from rare.components.tabs.games.game_widgets.base_uninstalled_widget import (
    BaseUninstalledWidget,
)

logger = getLogger("Game")


class UninstalledListWidget(BaseUninstalledWidget):
    def __init__(self, core: LegendaryCore, game, pixmap):
        super(UninstalledListWidget, self).__init__(game, core, pixmap)
        self.setFrameStyle(self.StyledPanel)
        layout = QHBoxLayout()
        self.setLayout(layout)
        layout.addWidget(self.image)

        self.child_layout = QVBoxLayout()
        layout.addLayout(self.child_layout)

        self.title_label = QLabel(f"<h2>{self.game.app_title}</h2>")
        self.app_name_label = QLabel(f"App Name: {self.game.app_name}")
        self.install_button = QPushButton(self.tr("Install"))
        self.install_button.setFixedWidth(120)
        self.install_button.clicked.connect(self.install)

        self.child_layout.addWidget(self.title_label)
        self.child_layout.addWidget(self.app_name_label)
        self.child_layout.addWidget(self.install_button)

        layout.setAlignment(Qt.AlignLeft)
        self.child_layout.setAlignment(Qt.AlignTop)



import os
import shutil
from pathlib import Path

from PyQt5.QtCore import QStandardPaths

resources_path = Path(__file__).absolute().parent.parent.joinpath("resources")

# lk: delete old Rare directories
for old_dir in [
    Path(QStandardPaths.writableLocation(QStandardPaths.CacheLocation), "rare").joinpath("tmp"),
    Path(QStandardPaths.writableLocation(QStandardPaths.DataLocation), "rare").joinpath("images"),
    Path(QStandardPaths.writableLocation(QStandardPaths.CacheLocation), "rare"),
    Path(QStandardPaths.writableLocation(QStandardPaths.DataLocation), "rare"),
]:
    if old_dir.exists():
        # lk: case-sensitive matching on Winblows
        if old_dir.stem in os.listdir(old_dir.parent):
            shutil.rmtree(old_dir, ignore_errors=True)


# lk: TempLocation doesn't depend on OrganizationName or ApplicationName
# lk: so it is fine to use it before initializing the QApplication
def lock_file() -> Path:
    return Path(QStandardPaths.writableLocation(QStandardPaths.TempLocation), "Rare.lock")


def data_dir() -> Path:
    return Path(QStandardPaths.writableLocation(QStandardPaths.DataLocation))


def cache_dir() -> Path:
    return Path(QStandardPaths.writableLocation(QStandardPaths.CacheLocation))


def image_dir() -> Path:
    return data_dir().joinpath("images")


def log_dir() -> Path:
    return cache_dir().joinpath("logs")


def tmp_dir() -> Path:
    return cache_dir().joinpath("tmp")


def create_dirs() -> None:
    for path in (data_dir(), cache_dir(), image_dir(), log_dir(), tmp_dir()):
        if not path.exists():
            path.mkdir(parents=True)



from rare.components.tabs.settings.widgets.linux import LinuxSettings
from rare.utils.extra_widgets import SideTabWidget
from .about import About
from .default_game_settings import DefaultGameSettings
from .legendary import LegendarySettings
from .rare import RareSettings


class SettingsTab(SideTabWidget):
    def __init__(self, parent=None):
        super(SettingsTab, self).__init__(parent=parent)
        about_tab = 3
        self.rare_settings = RareSettings()
        self.addTab(self.rare_settings, "Rare")

        self.addTab(LegendarySettings(), "Legendary")

        self.addTab(DefaultGameSettings(True, self), self.tr("Default Settings"))

        self.about = About()
        self.addTab(self.about, "About", "About")
        self.about.update_available_ready.connect(
            lambda: self.tabBar().setTabText(about_tab, "About (!)")
        )

        self.setCurrentIndex(0)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'rare/ui/components/dialogs/install_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.7
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_InstallDialog(object):
    def setupUi(self, InstallDialog):
        InstallDialog.setObjectName("InstallDialog")
        InstallDialog.resize(383, 436)
        InstallDialog.setWindowTitle("Rare")
        self.install_dialog_layout = QtWidgets.QFormLayout(InstallDialog)
        self.install_dialog_layout.setLabelAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.install_dialog_layout.setObjectName("install_dialog_layout")
        self.install_dialog_label = QtWidgets.QLabel(InstallDialog)
        self.install_dialog_label.setObjectName("install_dialog_label")
        self.install_dialog_layout.setWidget(0, QtWidgets.QFormLayout.SpanningRole, self.install_dialog_label)
        self.install_dir_label = QtWidgets.QLabel(InstallDialog)
        self.install_dir_label.setObjectName("install_dir_label")
        self.install_dialog_layout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.install_dir_label)
        self.install_dir_layout = QtWidgets.QHBoxLayout()
        self.install_dir_layout.setObjectName("install_dir_layout")
        self.install_dialog_layout.setLayout(1, QtWidgets.QFormLayout.FieldRole, self.install_dir_layout)
        self.platform_label = QtWidgets.QLabel(InstallDialog)
        self.platform_label.setObjectName("platform_label")
        self.install_dialog_layout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.platform_label)
        self.platform_combo_box = QtWidgets.QComboBox(InstallDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.platform_combo_box.sizePolicy().hasHeightForWidth())
        self.platform_combo_box.setSizePolicy(sizePolicy)
        self.platform_combo_box.setObjectName("platform_combo_box")
        self.install_dialog_layout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.platform_combo_box)
        self.shortcut_lbl = QtWidgets.QLabel(InstallDialog)
        self.shortcut_lbl.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.shortcut_lbl.setObjectName("shortcut_lbl")
        self.install_dialog_layout.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.shortcut_lbl)
        self.shortcut_cb = QtWidgets.QCheckBox(InstallDialog)
        self.shortcut_cb.setText("")
        self.shortcut_cb.setObjectName("shortcut_cb")
        self.install_dialog_layout.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.shortcut_cb)
        self.sdl_list_label = QtWidgets.QLabel(InstallDialog)
        self.sdl_list_label.setObjectName("sdl_list_label")
        self.install_dialog_layout.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.sdl_list_label)
        self.sdl_list_frame = QtWidgets.QFrame(InstallDialog)
        self.sdl_list_frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.sdl_list_frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.sdl_list_frame.setObjectName("sdl_list_frame")
        self.sdl_list_layout = QtWidgets.QVBoxLayout(self.sdl_list_frame)
        self.sdl_list_layout.setSpacing(0)
        self.sdl_list_layout.setObjectName("sdl_list_layout")
        self.install_dialog_layout.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.sdl_list_frame)
        self.collapsible_layout = QtWidgets.QVBoxLayout()
        self.collapsible_layout.setObjectName("collapsible_layout")
        self.advanced_layout = QtWidgets.QFormLayout()
        self.advanced_layout.setLabelAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.advanced_layout.setObjectName("advanced_layout")
        self.max_workers_label = QtWidgets.QLabel(InstallDialog)
        self.max_workers_label.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.max_workers_label.setObjectName("max_workers_label")
        self.advanced_layout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.max_workers_label)
        self.max_workers_layout = QtWidgets.QHBoxLayout()
        self.max_workers_layout.setObjectName("max_workers_layout")
        self.max_workers_spin = QtWidgets.QSpinBox(InstallDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.max_workers_spin.sizePolicy().hasHeightForWidth())
        self.max_workers_spin.setSizePolicy(sizePolicy)
        self.max_workers_spin.setObjectName("max_workers_spin")
        self.max_workers_layout.addWidget(self.max_workers_spin)
        self.max_workers_info_label = QtWidgets.QLabel(InstallDialog)
        font = QtGui.QFont()
        font.setItalic(True)
        self.max_workers_info_label.setFont(font)
        self.max_workers_info_label.setObjectName("max_workers_info_label")
        self.max_workers_layout.addWidget(self.max_workers_info_label)
        self.advanced_layout.setLayout(0, QtWidgets.QFormLayout.FieldRole, self.max_workers_layout)
        self.max_memory_label = QtWidgets.QLabel(InstallDialog)
        self.max_memory_label.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.max_memory_label.setObjectName("max_memory_label")
        self.advanced_layout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.max_memory_label)
        self.max_memory_layout = QtWidgets.QHBoxLayout()
        self.max_memory_layout.setObjectName("max_memory_layout")
        self.max_memory_spin = QtWidgets.QSpinBox(InstallDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.max_memory_spin.sizePolicy().hasHeightForWidth())
        self.max_memory_spin.setSizePolicy(sizePolicy)
        self.max_memory_spin.setMinimum(0)
        self.max_memory_spin.setMaximum(10240)
        self.max_memory_spin.setSingleStep(128)
        self.max_memory_spin.setProperty("value", 1024)
        self.max_memory_spin.setObjectName("max_memory_spin")
        self.max_memory_layout.addWidget(self.max_memory_spin)
        self.max_memory_info_label = QtWidgets.QLabel(InstallDialog)
        font = QtGui.QFont()
        font.setItalic(True)
        self.max_memory_info_label.setFont(font)
        self.max_memory_info_label.setObjectName("max_memory_info_label")
        self.max_memory_layout.addWidget(self.max_memory_info_label)
        self.advanced_layout.setLayout(1, QtWidgets.QFormLayout.FieldRole, self.max_memory_layout)
        self.install_prereqs_lbl = QtWidgets.QLabel(InstallDialog)
        self.install_prereqs_lbl.setObjectName("install_prereqs_lbl")
        self.advanced_layout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.install_prereqs_lbl)
        self.install_prereqs_check = QtWidgets.QCheckBox(InstallDialog)
        font = QtGui.QFont()
        font.setItalic(True)
        self.install_prereqs_check.setFont(font)
        self.install_prereqs_check.setText("")
        self.install_prereqs_check.setChecked(False)
        self.install_prereqs_check.setObjectName("install_prereqs_check")
        self.advanced_layout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.install_prereqs_check)
        self.dl_optimizations_label = QtWidgets.QLabel(InstallDialog)
        self.dl_optimizations_label.setObjectName("dl_optimizations_label")
        self.advanced_layout.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.dl_optimizations_label)
        self.dl_optimizations_check = QtWidgets.QCheckBox(InstallDialog)
        self.dl_optimizations_check.setText("")
        self.dl_optimizations_check.setChecked(False)
        self.dl_optimizations_check.setObjectName("dl_optimizations_check")
        self.advanced_layout.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.dl_optimizations_check)
        self.force_download_label = QtWidgets.QLabel(InstallDialog)
        self.force_download_label.setObjectName("force_download_label")
        self.advanced_layout.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.force_download_label)
        self.force_download_check = QtWidgets.QCheckBox(InstallDialog)
        self.force_download_check.setText("")
        self.force_download_check.setObjectName("force_download_check")
        self.advanced_layout.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.force_download_check)
        self.ignore_space_label = QtWidgets.QLabel(InstallDialog)
        self.ignore_space_label.setObjectName("ignore_space_label")
        self.advanced_layout.setWidget(5, QtWidgets.QFormLayout.LabelRole, self.ignore_space_label)
        self.ignore_space_check = QtWidgets.QCheckBox(InstallDialog)
        font = QtGui.QFont()
        font.setItalic(True)
        self.ignore_space_check.setFont(font)
        self.ignore_space_check.setObjectName("ignore_space_check")
        self.advanced_layout.setWidget(5, QtWidgets.QFormLayout.FieldRole, self.ignore_space_check)
        self.download_only_label = QtWidgets.QLabel(InstallDialog)
        self.download_only_label.setObjectName("download_only_label")
        self.advanced_layout.setWidget(6, QtWidgets.QFormLayout.LabelRole, self.download_only_label)
        self.download_only_check = QtWidgets.QCheckBox(InstallDialog)
        font = QtGui.QFont()
        font.setItalic(True)
        self.download_only_check.setFont(font)
        self.download_only_check.setObjectName("download_only_check")
        self.advanced_layout.setWidget(6, QtWidgets.QFormLayout.FieldRole, self.download_only_check)
        self.collapsible_layout.addLayout(self.advanced_layout)
        self.install_dialog_layout.setLayout(5, QtWidgets.QFormLayout.SpanningRole, self.collapsible_layout)
        self.download_size_label = QtWidgets.QLabel(InstallDialog)
        self.download_size_label.setObjectName("download_size_label")
        self.install_dialog_layout.setWidget(6, QtWidgets.QFormLayout.LabelRole, self.download_size_label)
        self.download_size_info_label = QtWidgets.QLabel(InstallDialog)
        font = QtGui.QFont()
        font.setItalic(True)
        self.download_size_info_label.setFont(font)
        self.download_size_info_label.setObjectName("download_size_info_label")
        self.install_dialog_layout.setWidget(6, QtWidgets.QFormLayout.FieldRole, self.download_size_info_label)
        self.install_size_label = QtWidgets.QLabel(InstallDialog)
        self.install_size_label.setObjectName("install_size_label")
        self.install_dialog_layout.setWidget(7, QtWidgets.QFormLayout.LabelRole, self.install_size_label)
        self.install_size_info_label = QtWidgets.QLabel(InstallDialog)
        font = QtGui.QFont()
        font.setItalic(True)
        self.install_size_info_label.setFont(font)
        self.install_size_info_label.setWordWrap(True)
        self.install_size_info_label.setObjectName("install_size_info_label")
        self.install_dialog_layout.setWidget(7, QtWidgets.QFormLayout.FieldRole, self.install_size_info_label)
        self.warn_label = QtWidgets.QLabel(InstallDialog)
        self.warn_label.setObjectName("warn_label")
        self.install_dialog_layout.setWidget(8, QtWidgets.QFormLayout.LabelRole, self.warn_label)
        self.warn_message = QtWidgets.QLabel(InstallDialog)
        font = QtGui.QFont()
        font.setItalic(True)
        self.warn_message.setFont(font)
        self.warn_message.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.warn_message.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.warn_message.setWordWrap(True)
        self.warn_message.setObjectName("warn_message")
        self.install_dialog_layout.setWidget(8, QtWidgets.QFormLayout.FieldRole, self.warn_message)
        self.button_layout = QtWidgets.QHBoxLayout()
        self.button_layout.setObjectName("button_layout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.button_layout.addItem(spacerItem)
        self.cancel_button = QtWidgets.QPushButton(InstallDialog)
        self.cancel_button.setObjectName("cancel_button")
        self.button_layout.addWidget(self.cancel_button)
        self.verify_button = QtWidgets.QPushButton(InstallDialog)
        self.verify_button.setObjectName("verify_button")
        self.button_layout.addWidget(self.verify_button)
        self.install_button = QtWidgets.QPushButton(InstallDialog)
        self.install_button.setObjectName("install_button")
        self.button_layout.addWidget(self.install_button)
        self.install_dialog_layout.setLayout(9, QtWidgets.QFormLayout.SpanningRole, self.button_layout)

        self.retranslateUi(InstallDialog)
        QtCore.QMetaObject.connectSlotsByName(InstallDialog)

    def retranslateUi(self, InstallDialog):
        _translate = QtCore.QCoreApplication.translate
        self.install_dialog_label.setText(_translate("InstallDialog", "error"))
        self.install_dir_label.setText(_translate("InstallDialog", "Install directory"))
        self.platform_label.setText(_translate("InstallDialog", "Platform"))
        self.shortcut_lbl.setText(_translate("InstallDialog", "Create shortcut"))
        self.sdl_list_label.setText(_translate("InstallDialog", "Optional packs"))
        self.max_workers_label.setText(_translate("InstallDialog", "Max workers"))
        self.max_workers_info_label.setText(_translate("InstallDialog", "Less is slower. (0: Default)"))
        self.max_memory_label.setText(_translate("InstallDialog", "Max shared memory"))
        self.max_memory_spin.setSuffix(_translate("InstallDialog", "MiB"))
        self.max_memory_info_label.setText(_translate("InstallDialog", "Less is slower (0: Default)"))
        self.install_prereqs_lbl.setText(_translate("InstallDialog", "Install prerequisites"))
        self.dl_optimizations_label.setText(_translate("InstallDialog", "Enable reordering"))
        self.force_download_label.setText(_translate("InstallDialog", "Force redownload"))
        self.ignore_space_label.setText(_translate("InstallDialog", "Ignore free space"))
        self.ignore_space_check.setText(_translate("InstallDialog", "Use with caution!"))
        self.download_only_label.setText(_translate("InstallDialog", "Download only"))
        self.download_only_check.setText(_translate("InstallDialog", "Do not try to install."))
        self.download_size_label.setText(_translate("InstallDialog", "Download size"))
        self.download_size_info_label.setText(_translate("InstallDialog", "Click verify..."))
        self.install_size_label.setText(_translate("InstallDialog", "Total install size"))
        self.install_size_info_label.setText(_translate("InstallDialog", "Click verify..."))
        self.warn_label.setText(_translate("InstallDialog", "Warning"))
        self.warn_message.setText(_translate("InstallDialog", "None"))
        self.cancel_button.setText(_translate("InstallDialog", "Cancel"))
        self.verify_button.setText(_translate("InstallDialog", "Verify"))
        self.install_button.setText(_translate("InstallDialog", "Install"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    InstallDialog = QtWidgets.QDialog()
    ui = Ui_InstallDialog()
    ui.setupUi(InstallDialog)
    InstallDialog.show()
    sys.exit(app.exec_())



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'rare/ui/components/tabs/games/game_info/game_info.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_GameInfo(object):
    def setupUi(self, GameInfo):
        GameInfo.setObjectName("GameInfo")
        GameInfo.resize(791, 583)
        self.layout_game_info = QtWidgets.QHBoxLayout(GameInfo)
        self.layout_game_info.setObjectName("layout_game_info")
        self.layout_game_info_form = QtWidgets.QGridLayout()
        self.layout_game_info_form.setContentsMargins(6, 6, 6, 6)
        self.layout_game_info_form.setSpacing(12)
        self.layout_game_info_form.setObjectName("layout_game_info_form")
        spacerItem = QtWidgets.QSpacerItem(0, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.layout_game_info_form.addItem(spacerItem, 8, 1, 1, 1)
        self.dev = QtWidgets.QLabel(GameInfo)
        self.dev.setText("error")
        self.dev.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByMouse)
        self.dev.setObjectName("dev")
        self.layout_game_info_form.addWidget(self.dev, 0, 1, 1, 1)
        self.lbl_dev = QtWidgets.QLabel(GameInfo)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lbl_dev.sizePolicy().hasHeightForWidth())
        self.lbl_dev.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.lbl_dev.setFont(font)
        self.lbl_dev.setObjectName("lbl_dev")
        self.layout_game_info_form.addWidget(self.lbl_dev, 0, 0, 1, 1, QtCore.Qt.AlignRight)
        self.app_name = QtWidgets.QLabel(GameInfo)
        self.app_name.setText("error")
        self.app_name.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByMouse)
        self.app_name.setObjectName("app_name")
        self.layout_game_info_form.addWidget(self.app_name, 1, 1, 1, 1)
        self.install_path = QtWidgets.QLabel(GameInfo)
        self.install_path.setText("error")
        self.install_path.setWordWrap(True)
        self.install_path.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByMouse)
        self.install_path.setObjectName("install_path")
        self.layout_game_info_form.addWidget(self.install_path, 5, 1, 1, 1)
        self.lbl_platform = QtWidgets.QLabel(GameInfo)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.lbl_platform.setFont(font)
        self.lbl_platform.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.lbl_platform.setObjectName("lbl_platform")
        self.layout_game_info_form.addWidget(self.lbl_platform, 6, 0, 1, 1)
        self.version = QtWidgets.QLabel(GameInfo)
        self.version.setText("error")
        self.version.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByMouse)
        self.version.setObjectName("version")
        self.layout_game_info_form.addWidget(self.version, 2, 1, 1, 1)
        self.lbl_install_path = QtWidgets.QLabel(GameInfo)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lbl_install_path.sizePolicy().hasHeightForWidth())
        self.lbl_install_path.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.lbl_install_path.setFont(font)
        self.lbl_install_path.setObjectName("lbl_install_path")
        self.layout_game_info_form.addWidget(self.lbl_install_path, 5, 0, 1, 1, QtCore.Qt.AlignRight)
        self.lbl_install_size = QtWidgets.QLabel(GameInfo)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lbl_install_size.sizePolicy().hasHeightForWidth())
        self.lbl_install_size.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.lbl_install_size.setFont(font)
        self.lbl_install_size.setObjectName("lbl_install_size")
        self.layout_game_info_form.addWidget(self.lbl_install_size, 4, 0, 1, 1, QtCore.Qt.AlignRight)
        spacerItem1 = QtWidgets.QSpacerItem(20, 0, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.layout_game_info_form.addItem(spacerItem1, 8, 0, 1, 1)
        self.platform = QtWidgets.QLabel(GameInfo)
        self.platform.setText("error")
        self.platform.setObjectName("platform")
        self.layout_game_info_form.addWidget(self.platform, 6, 1, 1, 1)
        self.game_actions_stack = QtWidgets.QStackedWidget(GameInfo)
        self.game_actions_stack.setMinimumSize(QtCore.QSize(250, 0))
        self.game_actions_stack.setObjectName("game_actions_stack")
        self.installed_page = QtWidgets.QWidget()
        self.installed_page.setObjectName("installed_page")
        self.installed_layout = QtWidgets.QVBoxLayout(self.installed_page)
        self.installed_layout.setContentsMargins(0, 0, 0, 0)
        self.installed_layout.setObjectName("installed_layout")
        self.verify_widget = QtWidgets.QStackedWidget(self.installed_page)
        self.verify_widget.setObjectName("verify_widget")
        self.page_verify_button = QtWidgets.QWidget()
        self.page_verify_button.setObjectName("page_verify_button")
        self.layout_verify_button = QtWidgets.QVBoxLayout(self.page_verify_button)
        self.layout_verify_button.setContentsMargins(0, 0, 0, 0)
        self.layout_verify_button.setSpacing(0)
        self.layout_verify_button.setObjectName("layout_verify_button")
        self.verify_button = QtWidgets.QPushButton(self.page_verify_button)
        self.verify_button.setObjectName("verify_button")
        self.layout_verify_button.addWidget(self.verify_button)
        self.verify_widget.addWidget(self.page_verify_button)
        self.page_verify_progress = QtWidgets.QWidget()
        self.page_verify_progress.setObjectName("page_verify_progress")
        self.layout_verify_progress = QtWidgets.QVBoxLayout(self.page_verify_progress)
        self.layout_verify_progress.setContentsMargins(0, 0, 0, 0)
        self.layout_verify_progress.setSpacing(0)
        self.layout_verify_progress.setObjectName("layout_verify_progress")
        self.verify_progress = QtWidgets.QProgressBar(self.page_verify_progress)
        self.verify_progress.setProperty("value", 24)
        self.verify_progress.setObjectName("verify_progress")
        self.layout_verify_progress.addWidget(self.verify_progress)
        self.verify_widget.addWidget(self.page_verify_progress)
        self.installed_layout.addWidget(self.verify_widget)
        self.repair_button = QtWidgets.QPushButton(self.installed_page)
        self.repair_button.setObjectName("repair_button")
        self.installed_layout.addWidget(self.repair_button)
        self.move_stack = QtWidgets.QStackedWidget(self.installed_page)
        self.move_stack.setMinimumSize(QtCore.QSize(0, 20))
        self.move_stack.setObjectName("move_stack")
        self.page = QtWidgets.QWidget()
        self.page.setObjectName("page")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.page)
        self.horizontalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.move_button = QtWidgets.QToolButton(self.page)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.move_button.sizePolicy().hasHeightForWidth())
        self.move_button.setSizePolicy(sizePolicy)
        self.move_button.setPopupMode(QtWidgets.QToolButton.InstantPopup)
        self.move_button.setObjectName("move_button")
        self.horizontalLayout_2.addWidget(self.move_button)
        self.move_stack.addWidget(self.page)
        self.page_2 = QtWidgets.QWidget()
        self.page_2.setObjectName("page_2")
        self.move_stack.addWidget(self.page_2)
        self.installed_layout.addWidget(self.move_stack)
        self.uninstall_button = QtWidgets.QPushButton(self.installed_page)
        self.uninstall_button.setStyleSheet("")
        self.uninstall_button.setObjectName("uninstall_button")
        self.installed_layout.addWidget(self.uninstall_button)
        self.game_actions_stack.addWidget(self.installed_page)
        self.uninstalled_page = QtWidgets.QWidget()
        self.uninstalled_page.setObjectName("uninstalled_page")
        self.uninstalled_layout = QtWidgets.QVBoxLayout(self.uninstalled_page)
        self.uninstalled_layout.setObjectName("uninstalled_layout")
        self.install_button = QtWidgets.QPushButton(self.uninstalled_page)
        self.install_button.setStyleSheet("")
        self.install_button.setObjectName("install_button")
        self.uninstalled_layout.addWidget(self.install_button)
        self.game_actions_stack.addWidget(self.uninstalled_page)
        self.layout_game_info_form.addWidget(self.game_actions_stack, 7, 1, 1, 1, QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.install_size = QtWidgets.QLabel(GameInfo)
        self.install_size.setText("error")
        self.install_size.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByMouse)
        self.install_size.setObjectName("install_size")
        self.layout_game_info_form.addWidget(self.install_size, 4, 1, 1, 1)
        self.lbl_version = QtWidgets.QLabel(GameInfo)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lbl_version.sizePolicy().hasHeightForWidth())
        self.lbl_version.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.lbl_version.setFont(font)
        self.lbl_version.setObjectName("lbl_version")
        self.layout_game_info_form.addWidget(self.lbl_version, 2, 0, 1, 1, QtCore.Qt.AlignRight)
        self.lbl_game_actions = QtWidgets.QLabel(GameInfo)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lbl_game_actions.sizePolicy().hasHeightForWidth())
        self.lbl_game_actions.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.lbl_game_actions.setFont(font)
        self.lbl_game_actions.setObjectName("lbl_game_actions")
        self.layout_game_info_form.addWidget(self.lbl_game_actions, 7, 0, 1, 1, QtCore.Qt.AlignRight)
        self.lbl_grade = QtWidgets.QLabel(GameInfo)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.lbl_grade.setFont(font)
        self.lbl_grade.setObjectName("lbl_grade")
        self.layout_game_info_form.addWidget(self.lbl_grade, 3, 0, 1, 1, QtCore.Qt.AlignRight)
        self.lbl_app_name = QtWidgets.QLabel(GameInfo)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lbl_app_name.sizePolicy().hasHeightForWidth())
        self.lbl_app_name.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.lbl_app_name.setFont(font)
        self.lbl_app_name.setObjectName("lbl_app_name")
        self.layout_game_info_form.addWidget(self.lbl_app_name, 1, 0, 1, 1, QtCore.Qt.AlignRight)
        self.grade = QtWidgets.QLabel(GameInfo)
        self.grade.setText("error")
        self.grade.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByMouse)
        self.grade.setObjectName("grade")
        self.layout_game_info_form.addWidget(self.grade, 3, 1, 1, 1)
        self.layout_game_info.addLayout(self.layout_game_info_form)

        self.retranslateUi(GameInfo)
        self.game_actions_stack.setCurrentIndex(0)
        self.verify_widget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(GameInfo)

    def retranslateUi(self, GameInfo):
        _translate = QtCore.QCoreApplication.translate
        GameInfo.setWindowTitle(_translate("GameInfo", "Game Info"))
        self.lbl_dev.setText(_translate("GameInfo", "Developer"))
        self.lbl_platform.setText(_translate("GameInfo", "Platform"))
        self.lbl_install_path.setText(_translate("GameInfo", "Installation Path"))
        self.lbl_install_size.setText(_translate("GameInfo", "Installation Size"))
        self.verify_button.setText(_translate("GameInfo", "Verify Installation"))
        self.repair_button.setText(_translate("GameInfo", "Repair Installation"))
        self.move_button.setText(_translate("GameInfo", "Move Installation"))
        self.uninstall_button.setText(_translate("GameInfo", "Uninstall Game"))
        self.install_button.setText(_translate("GameInfo", "Install Game"))
        self.lbl_version.setText(_translate("GameInfo", "Version"))
        self.lbl_game_actions.setText(_translate("GameInfo", "Actions"))
        self.lbl_grade.setText(_translate("GameInfo", "ProtonDB Grade"))
        self.lbl_app_name.setText(_translate("GameInfo", "Application Name"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    GameInfo = QtWidgets.QWidget()
    ui = Ui_GameInfo()
    ui.setupUi(GameInfo)
    GameInfo.show()
    sys.exit(app.exec_())



import datetime
from logging import getLogger
from typing import List, Dict, Union, Set

from PyQt5.QtCore import QThread, pyqtSignal, QSettings, pyqtSlot
from PyQt5.QtWidgets import (
    QWidget,
    QMessageBox,
    QVBoxLayout,
    QLabel,
    QPushButton,
    QGroupBox,
)
from legendary.core import LegendaryCore
from legendary.models.game import Game, InstalledGame

from rare.components.dialogs.install_dialog import InstallDialog
from rare.components.tabs.downloads.dl_queue_widget import DlQueueWidget, DlWidget
from rare.components.tabs.downloads.download_thread import DownloadThread
from rare.lgndr.downloading import UIUpdate
from rare.models.install import InstallOptionsModel, InstallQueueItemModel
from rare.shared import LegendaryCoreSingleton, GlobalSignalsSingleton
from rare.ui.components.tabs.downloads.downloads_tab import Ui_DownloadsTab
from rare.utils.misc import get_size, create_desktop_link

logger = getLogger("Download")


class DownloadsTab(QWidget, Ui_DownloadsTab):
    thread: QThread
    dl_queue: List[InstallQueueItemModel] = []
    dl_status = pyqtSignal(int)

    def __init__(self, updates: Set):
        super(DownloadsTab, self).__init__()
        self.setupUi(self)
        self.core = LegendaryCoreSingleton()
        self.signals = GlobalSignalsSingleton()
        self.active_game: Game = None
        self.analysis = None

        self.kill_button.clicked.connect(self.stop_download)

        self.queue_widget = DlQueueWidget()
        self.queue_widget.update_list.connect(self.update_dl_queue)
        self.queue_scroll_contents_layout.addWidget(self.queue_widget)

        self.updates = QGroupBox(self.tr("Updates"))
        self.updates.setObjectName("updates_group")
        self.update_layout = QVBoxLayout(self.updates)
        self.queue_scroll_contents_layout.addWidget(self.updates)

        self.update_widgets: Dict[str, UpdateWidget] = {}

        self.update_text = QLabel(self.tr("No updates available"))
        self.update_layout.addWidget(self.update_text)
        self.update_text.setVisible(len(updates) == 0)

        for app_name in updates:
            self.add_update(app_name)

        self.queue_widget.item_removed.connect(self.queue_item_removed)

        self.signals.install_game.connect(self.get_install_options)
        self.signals.game_uninstalled.connect(self.queue_item_removed)
        self.signals.game_uninstalled.connect(self.remove_update)

        self.signals.add_download.connect(
            lambda app_name: self.add_update(app_name)
        )
        self.signals.game_uninstalled.connect(self.game_uninstalled)

        self.reset_infos()

    def queue_item_removed(self, app_name):
        if w := self.update_widgets.get(app_name):
            w.update_button.setDisabled(False)
            w.update_with_settings.setDisabled(False)

    def add_update(self, app_name: str):
        if old_widget := self.update_widgets.get(app_name, False):
            old_widget.deleteLater()
            self.update_widgets.pop(app_name)
        widget = UpdateWidget(self.core, app_name, self)
        self.update_layout.addWidget(widget)
        self.update_widgets[app_name] = widget
        widget.update_signal.connect(self.get_install_options)
        if QSettings().value("auto_update", False, bool):
            self.get_install_options(
                InstallOptionsModel(app_name=app_name, update=True, silent=True)
            )
            widget.update_button.setDisabled(True)
        self.update_text.setVisible(False)

    def game_uninstalled(self, app_name):
        # game in dl_queue
        for i, item in enumerate(self.dl_queue):
            if item.options.app_name == app_name:
                self.dl_queue.pop(i)
                self.queue_widget.update_queue(self.dl_queue)
            break

        # if game is updating
        if self.active_game and self.active_game.app_name == app_name:
            self.stop_download()

        # game has available update
        if app_name in self.update_widgets.keys():
            self.remove_update(app_name)

    def remove_update(self, app_name):
        if w := self.update_widgets.get(app_name):
            w.deleteLater()
            self.update_widgets.pop(app_name)

        if len(self.update_widgets) == 0:
            self.update_text.setVisible(True)

        self.signals.update_download_tab_text.emit()

    def update_dl_queue(self, dl_queue):
        self.dl_queue = dl_queue

    def stop_download(self):
        self.thread.kill()
        self.kill_button.setEnabled(False)

    def install_game(self, queue_item: InstallQueueItemModel):
        if self.active_game is None:
            self.start_installation(queue_item)
        else:
            self.dl_queue.append(queue_item)
            self.queue_widget.update_queue(self.dl_queue)

    def start_installation(self, queue_item: InstallQueueItemModel):
        if self.dl_queue:
            self.dl_queue.pop(0)
            self.queue_widget.update_queue(self.dl_queue)
        self.active_game = queue_item.download.game
        self.thread = DownloadThread(self.core, queue_item)
        self.thread.ret_status.connect(self.status)
        self.thread.ui_update.connect(self.progress_update)
        self.thread.start()
        self.kill_button.setDisabled(False)
        self.analysis = queue_item.download.analysis
        self.dl_name.setText(self.active_game.app_title)

        self.signals.installation_started.emit(self.active_game.app_name)

    @pyqtSlot(DownloadThread.ReturnStatus)
    def status(self, result: DownloadThread.ReturnStatus):
        if result.ret_code == result.ReturnCode.FINISHED:
            if result.shortcuts:
                if not create_desktop_link(result.app_name, self.core, "desktop"):
                    # maybe add it to download summary, to show in finished downloads
                    pass
                else:
                    logger.info("Desktop shortcut written")

            self.dl_name.setText(self.tr("Download finished. Reload library"))
            logger.info(f"Download finished: {self.active_game.app_title}")

            game = self.active_game
            self.active_game = None

            if self.dl_queue:
                if self.dl_queue[0].download.game.app_name == game.app_name:
                    self.dl_queue.pop(0)
                    self.queue_widget.update_queue(self.dl_queue)

            if game.app_name in self.update_widgets.keys():
                igame = self.core.get_installed_game(game.app_name)
                if (
                    self.core.get_asset(
                        game.app_name, igame.platform, False
                    ).build_version
                    == igame.version
                ):
                    self.remove_update(game.app_name)

            self.signals.send_notification.emit(game.app_title)
            self.signals.update_gamelist.emit([game.app_name])
            self.signals.update_download_tab_text.emit()

            self.signals.installation_finished.emit(True, game.app_name)

            self.reset_infos()

            if len(self.dl_queue) != 0:
                self.start_installation(self.dl_queue[0])
            else:
                self.queue_widget.update_queue(self.dl_queue)

        elif result.ret_code == result.ReturnCode.ERROR:
            QMessageBox.warning(self, self.tr("Error"), f"Download error: {result.message}")

        elif result.ret_code == result.ReturnCode.STOPPED:
            self.reset_infos()
            if w := self.update_widgets.get(self.active_game.app_name):
                w.update_button.setDisabled(False)
                w.update_with_settings.setDisabled(False)
            self.signals.installation_finished.emit(False, self.active_game.app_name)
            self.active_game = None
            if self.dl_queue:
                self.start_installation(self.dl_queue[0])

    def reset_infos(self):
        self.kill_button.setDisabled(True)
        self.dl_name.setText(self.tr("No active download"))
        self.progress_bar.setValue(0)
        self.dl_speed.setText("n/a")
        self.time_left.setText("n/a")
        self.cache_used.setText("n/a")
        self.downloaded.setText("n/a")
        self.analysis = None

    def progress_update(self, ui_update: UIUpdate):
        self.progress_bar.setValue(
            100 * ui_update.total_downloaded // self.analysis.dl_size
        )
        self.dl_speed.setText(f"{get_size(ui_update.download_speed)}/s")
        self.cache_used.setText(
            f"{get_size(ui_update.cache_usage) if ui_update.cache_usage > 1023 else '0KB'}"
        )
        self.downloaded.setText(
            f"{get_size(ui_update.total_downloaded)} / {get_size(self.analysis.dl_size)}"
        )
        self.time_left.setText(self.get_time(ui_update.estimated_time_left))
        self.signals.dl_progress.emit(
            100 * ui_update.total_downloaded // self.analysis.dl_size
        )

    def get_time(self, seconds: Union[int, float]) -> str:
        return str(datetime.timedelta(seconds=seconds))

    def on_install_dialog_closed(self, download_item: InstallQueueItemModel):
        if download_item:
            self.install_game(download_item)
            # lk: In case the download in comming from game verification/repair
            if w := self.update_widgets.get(download_item.options.app_name):
                w.update_button.setDisabled(True)
                w.update_with_settings.setDisabled(True)
            self.signals.set_main_tab_index.emit(1)
        else:
            if w := self.update_widgets.get(download_item.options.app_name):
                w.update_button.setDisabled(False)
                w.update_with_settings.setDisabled(False)

    def get_install_options(self, options: InstallOptionsModel):

        install_dialog = InstallDialog(
            InstallQueueItemModel(options=options),
            update=options.update,
            silent=options.silent,
            parent=self,
        )
        install_dialog.result_ready.connect(self.on_install_dialog_closed)
        install_dialog.execute()

    @property
    def is_download_active(self):
        return self.active_game is not None


class UpdateWidget(QWidget):
    update_signal = pyqtSignal(InstallOptionsModel)

    def __init__(self, core: LegendaryCore, app_name: str, parent):
        super(UpdateWidget, self).__init__(parent=parent)
        self.core = core
        self.game: Game = core.get_game(app_name)
        self.igame: InstalledGame = self.core.get_installed_game(app_name)

        layout = QVBoxLayout()
        self.title = QLabel(self.igame.title)
        layout.addWidget(self.title)

        self.update_button = QPushButton(self.tr("Update Game"))
        self.update_button.clicked.connect(lambda: self.update_game(True))
        self.update_with_settings = QPushButton("Update with settings")
        self.update_with_settings.clicked.connect(lambda: self.update_game(False))
        layout.addWidget(self.update_button)
        layout.addWidget(self.update_with_settings)
        layout.addWidget(
            QLabel(
                self.tr("Version: <b>")
                + self.igame.version
                + "</b> >> <b>"
                + self.game.app_version(self.igame.platform)
                + "</b>"
            )
        )

        self.setLayout(layout)

    def update_game(self, auto: bool):
        self.update_button.setDisabled(True)
        self.update_with_settings.setDisabled(True)
        self.update_signal.emit(
            InstallOptionsModel(app_name=self.igame.app_name, silent=auto)
        )  # True if settings



from logging import getLogger
from typing import List

from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import QSystemTrayIcon, QMenu, QAction

from rare.shared import GlobalSignalsSingleton
from rare.shared import LegendaryCoreSingleton
from rare.utils.meta import GameMeta

logger = getLogger("TrayIcon")


class TrayIcon(QSystemTrayIcon):
    def __init__(self, parent):
        super(TrayIcon, self).__init__(parent=parent)
        self.core = LegendaryCoreSingleton()

        self.setIcon(QIcon(":/images/Rare.png"))
        self.setVisible(True)
        self.setToolTip("Rare")

        self.menu = QMenu()

        self.start_rare = QAction("Rare")
        self.menu.addAction(self.start_rare)

        self.menu.addSeparator()
        self.text_action = QAction("Quick launch")
        self.text_action.setEnabled(False)
        self.menu.addAction(self.text_action)

        if len(installed := self.core.get_installed_list()) < 5:
            last_played = [GameMeta(i.app_name) for i in sorted(installed, key=lambda x: x.title)]
        elif games := sorted(
                parent.tab_widget.games_tab.game_utils.game_meta.get_games(),
                key=lambda x: x.last_played, reverse=True):
            last_played: List[GameMeta] = games[0:5]
        else:
            last_played = [GameMeta(i.app_name) for i in sorted(installed, key=lambda x: x.title)][0:5]

        self.game_actions: List[QAction] = []

        for game in last_played:
            a = QAction(self.core.get_game(game.app_name).app_title)
            a.setProperty("app_name", game.app_name)
            self.game_actions.append(a)
            a.triggered.connect(
                lambda: parent.tab_widget.games_tab.game_utils.prepare_launch(
                    self.sender().property("app_name"))
            )

        self.menu.addActions(self.game_actions)
        self.menu.addSeparator()

        self.exit_action = QAction(self.tr("Exit"))
        self.menu.addAction(self.exit_action)
        self.setContextMenu(self.menu)

        self.signals = GlobalSignalsSingleton()
        self.signals.game_uninstalled.connect(self.remove_button)

    def remove_button(self, app_name: str):
        if action := next((i for i in self.game_actions if i.property("app_name") == app_name), None):
            self.game_actions.remove(action)
            action.deleteLater()



import re
import shutil
from logging import getLogger
from typing import Dict, List

from PyQt5.QtCore import pyqtSignal, QSettings, QSize, Qt, QMimeData, pyqtSlot
from PyQt5.QtGui import QDrag, QDropEvent, QDragEnterEvent, QDragMoveEvent, QFont
from PyQt5.QtWidgets import QHBoxLayout, QLabel, QPushButton, QInputDialog, QFrame, QMessageBox, QSizePolicy, \
    QWidget, QScrollArea

from rare import shared
from rare.ui.components.tabs.settings.widgets.wrapper import Ui_WrapperSettings
from rare.utils import config_helper
from rare.utils.misc import icon

logger = getLogger("Wrapper Settings")

extra_wrapper_regex = {
    "proton": "\".*proton\" run",  # proton
    "mangohud": "mangohud"  # mangohud
}


class WrapperWidget(QFrame):
    delete_wrapper = pyqtSignal(str)

    def __init__(self, text: str, show_text=None, parent=None):
        super(WrapperWidget, self).__init__(parent=parent)
        if not show_text:
            show_text = text

        self.setFrameShape(QFrame.StyledPanel)
        self.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Fixed)

        self.text = text
        self.text_lbl = QLabel(show_text, parent=self)
        self.text_lbl.setFont(QFont("monospace"))
        self.image_lbl = QLabel(parent=self)
        self.image_lbl.setPixmap(icon("mdi.drag-vertical").pixmap(QSize(20, 20)))

        self.delete_button = QPushButton(icon("ei.remove"), "", parent=self)
        if show_text in extra_wrapper_regex.keys():
            self.delete_button.setDisabled(True)
            self.delete_button.setToolTip(self.tr("Disable it in settings"))
        self.delete_button.clicked.connect(self.delete)

        layout = QHBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)
        layout.addWidget(self.image_lbl)
        layout.addWidget(self.text_lbl)
        layout.addWidget(self.delete_button)
        self.setLayout(layout)

        # lk: set object names for the stylesheet
        self.setObjectName(type(self).__name__)
        self.delete_button.setObjectName(f"{self.objectName()}Button")

    def delete(self):
        self.delete_wrapper.emit(self.text)

    def mouseMoveEvent(self, e):
        if e.buttons() == Qt.LeftButton:
            drag = QDrag(self)
            mime = QMimeData()
            drag.setMimeData(mime)
            drag.exec_(Qt.MoveAction)


class WrapperSettings(QWidget, Ui_WrapperSettings):
    wrappers: Dict[str, WrapperWidget] = dict()
    app_name: str

    def __init__(self):
        super(WrapperSettings, self).__init__()
        self.setupUi(self)

        self.wrapper_scroll = QScrollArea(self.widget_stack)
        self.wrapper_scroll.setWidgetResizable(True)
        self.wrapper_scroll.setSizeAdjustPolicy(QScrollArea.AdjustToContents)
        self.wrapper_scroll.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.wrapper_scroll.setProperty("no_kinetic_scroll", True)
        self.scroll_content = WrapperContainer(
            save_cb=self.save, parent=self.wrapper_scroll
        )
        self.wrapper_scroll.setWidget(self.scroll_content)
        self.widget_stack.insertWidget(0, self.wrapper_scroll)

        self.core = shared.LegendaryCoreSingleton()

        self.add_button.clicked.connect(self.add_button_pressed)
        self.settings = QSettings()

        self.wrapper_scroll.horizontalScrollBar().rangeChanged.connect(self.adjust_scrollarea)

        # lk: set object names for the stylesheet
        self.setObjectName(type(self).__name__)
        self.no_wrapper_label.setObjectName(f"{self.objectName()}Label")
        self.wrapper_scroll.setObjectName(f"{self.objectName()}Scroll")
        self.wrapper_scroll.horizontalScrollBar().setObjectName(
            f"{self.wrapper_scroll.objectName()}Bar")
        self.wrapper_scroll.verticalScrollBar().setObjectName(
            f"{self.wrapper_scroll.objectName()}Bar")

    @pyqtSlot(int, int)
    def adjust_scrollarea(self, min: int, max: int):
        wrapper_widget = self.scroll_content.findChildren(WrapperWidget)[0]
        # lk: when the scrollbar is not visible, min and max are 0
        if max > min:
            self.wrapper_scroll.setMaximumHeight(
                wrapper_widget.sizeHint().height()
                + self.wrapper_scroll.rect().height() // 2
                - self.wrapper_scroll.contentsRect().height() // 2
                + self.scroll_content.layout().spacing()
                + self.wrapper_scroll.horizontalScrollBar().sizeHint().height()
            )
        else:
            self.wrapper_scroll.setMaximumHeight(
                wrapper_widget.sizeHint().height()
                + self.wrapper_scroll.rect().height()
                - self.wrapper_scroll.contentsRect().height()
            )

    def get_wrapper_string(self):
        return " ".join(self.get_wrapper_list())

    def get_wrapper_list(self):
        data: List[str] = []
        for w in self.wrappers.values():
            # Get the widget at each index in turn.
            try:
                data.append(w.text)
            except AttributeError:
                pass
        return data

    def add_button_pressed(self):
        wrapper, done = QInputDialog.getText(self, "Input Dialog", self.tr("Insert name of wrapper"))
        if not done:
            return
        self.add_wrapper(wrapper)

    def add_wrapper(self, text: str, from_load=False):
        if text == "mangohud" and self.wrappers.get("mangohud"):
            return
        show_text = text
        for key, extra_wrapper in extra_wrapper_regex.items():
            if re.match(extra_wrapper, text):
                show_text = key

        # validate
        if not text.strip():  # is empty
            return
        if not from_load:
            if self.wrappers.get(text):
                QMessageBox.warning(self, "Warning", self.tr("Wrapper is already in the list"))
                return

            if show_text != "proton" and not shutil.which(text.split()[0]):
                if QMessageBox.question(self, "Warning", self.tr("Wrapper is not in $PATH. Ignore? "),
                                        QMessageBox.Yes | QMessageBox.No, QMessageBox.No) == QMessageBox.No:
                    return

            if text == "proton":
                QMessageBox.warning(self, "Warning", self.tr("Do not insert proton manually. Add it in proton settings"))
                return

        self.widget_stack.setCurrentIndex(0)

        if widget := self.wrappers.get(show_text, None):
            widget.deleteLater()

        widget = WrapperWidget(text, show_text, self.scroll_content)
        self.scroll_content.layout().addWidget(widget)
        self.adjust_scrollarea(
            self.wrapper_scroll.horizontalScrollBar().minimum(),
            self.wrapper_scroll.horizontalScrollBar().maximum()
        )
        widget.delete_wrapper.connect(self.delete_wrapper)
        self.wrappers[show_text] = widget

        if not from_load:
            self.save()

    def delete_wrapper(self, text: str):
        widget = self.wrappers.get(text, None)
        if widget:
            self.wrappers.pop(text)
            widget.deleteLater()

        if not self.wrappers:
            self.wrapper_scroll.setMaximumHeight(self.label_page.sizeHint().height())
            self.widget_stack.setCurrentIndex(1)

        self.save()

    def save(self):
        # save wrappers twice, to support wrappers with spaces
        if len(self.wrappers) == 0:
            config_helper.remove_option(self.app_name, "wrapper")
            self.settings.remove(f"{self.app_name}/wrapper")
        else:
            config_helper.add_option(self.app_name, "wrapper", self.get_wrapper_string())
            self.settings.setValue(f"{self.app_name}/wrapper", self.get_wrapper_list())

    def load_settings(self, app_name: str):
        self.app_name = app_name
        for i in self.wrappers.values():
            i.deleteLater()
        self.wrappers.clear()

        wrappers = self.settings.value(f"{self.app_name}/wrapper", [], str)

        if not wrappers and (cfg := self.core.lgd.config.get(self.app_name, "wrapper", fallback="")):
            logger.info("Loading wrappers from legendary config")
            # no qt wrapper, but legendary wrapper, to have backward compatibility
            pattern = re.compile(r'''((?:[^ "']|"[^"]*"|'[^']*')+)''')
            wrappers = pattern.split(cfg)[1::2]

        for wrapper in wrappers:
            self.add_wrapper(wrapper, True)

        if not self.wrappers:
            self.wrapper_scroll.setMaximumHeight(self.label_page.sizeHint().height())
            self.widget_stack.setCurrentIndex(1)
        else:
            self.widget_stack.setCurrentIndex(0)

        self.save()


class WrapperContainer(QWidget):
    drag_widget: QWidget

    def __init__(self, save_cb, parent=None):
        super(WrapperContainer, self).__init__(parent=parent)
        self.setAcceptDrops(True)
        self.save = save_cb
        layout = QHBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)
        layout.setAlignment(Qt.AlignLeft | Qt.AlignTop)
        self.setLayout(layout)

        # lk: set object names for the stylesheet
        self.setObjectName(type(self).__name__)

    def dragEnterEvent(self, e: QDragEnterEvent):
        widget = e.source()
        self.drag_widget = widget
        e.accept()

    def _get_drop_index(self, x):
        drag_idx = self.layout().indexOf(self.drag_widget)

        if drag_idx > 0:
            prev_widget = self.layout().itemAt(drag_idx - 1).widget()
            if x < self.drag_widget.x() - prev_widget.width() // 2:
                return drag_idx - 1
        if drag_idx < self.layout().count() - 1:
            next_widget = self.layout().itemAt(drag_idx + 1).widget()
            if x > self.drag_widget.x() + self.drag_widget.width() + next_widget.width() // 2:
                return drag_idx + 1

        return drag_idx

    def dragMoveEvent(self, e: QDragMoveEvent) -> None:
        i = self._get_drop_index(e.pos().x())
        self.layout().insertWidget(i, self.drag_widget)

    def dropEvent(self, e: QDropEvent):
        pos = e.pos()
        widget = e.source()
        index = self._get_drop_index(pos.x())
        self.layout().insertWidget(index, widget)
        self.drag_widget = None
        e.accept()
        self.save()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'rare/ui/components/tabs/games/game_info/game_dlc.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_GameDlc(object):
    def setupUi(self, GameDlc):
        GameDlc.setObjectName("GameDlc")
        GameDlc.resize(287, 354)
        self.game_dlc_layout = QtWidgets.QVBoxLayout(GameDlc)
        self.game_dlc_layout.setObjectName("game_dlc_layout")
        self.installed_dlc_group = QtWidgets.QGroupBox(GameDlc)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.installed_dlc_group.sizePolicy().hasHeightForWidth())
        self.installed_dlc_group.setSizePolicy(sizePolicy)
        self.installed_dlc_group.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.installed_dlc_group.setObjectName("installed_dlc_group")
        self.installed_dlc_group_layout = QtWidgets.QVBoxLayout(self.installed_dlc_group)
        self.installed_dlc_group_layout.setObjectName("installed_dlc_group_layout")
        self.installed_dlc_label = QtWidgets.QLabel(self.installed_dlc_group)
        self.installed_dlc_label.setObjectName("installed_dlc_label")
        self.installed_dlc_group_layout.addWidget(self.installed_dlc_label)
        self.installed_dlc_scroll = QtWidgets.QScrollArea(self.installed_dlc_group)
        self.installed_dlc_scroll.setFocusPolicy(QtCore.Qt.WheelFocus)
        self.installed_dlc_scroll.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.installed_dlc_scroll.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.installed_dlc_scroll.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.installed_dlc_scroll.setWidgetResizable(True)
        self.installed_dlc_scroll.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.installed_dlc_scroll.setObjectName("installed_dlc_scroll")
        self.installed_dlc_contents = QtWidgets.QWidget()
        self.installed_dlc_contents.setGeometry(QtCore.QRect(0, 0, 255, 16))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.installed_dlc_contents.sizePolicy().hasHeightForWidth())
        self.installed_dlc_contents.setSizePolicy(sizePolicy)
        self.installed_dlc_contents.setObjectName("installed_dlc_contents")
        self.installed_dlc_contents_layout = QtWidgets.QVBoxLayout(self.installed_dlc_contents)
        self.installed_dlc_contents_layout.setContentsMargins(0, 0, 9, 0)
        self.installed_dlc_contents_layout.setObjectName("installed_dlc_contents_layout")
        self.installed_dlc_scroll.setWidget(self.installed_dlc_contents)
        self.installed_dlc_group_layout.addWidget(self.installed_dlc_scroll)
        self.game_dlc_layout.addWidget(self.installed_dlc_group)
        self.available_dlc_group = QtWidgets.QGroupBox(GameDlc)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.available_dlc_group.sizePolicy().hasHeightForWidth())
        self.available_dlc_group.setSizePolicy(sizePolicy)
        self.available_dlc_group.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.available_dlc_group.setObjectName("available_dlc_group")
        self.available_dlc_group_layout = QtWidgets.QVBoxLayout(self.available_dlc_group)
        self.available_dlc_group_layout.setObjectName("available_dlc_group_layout")
        self.available_dlc_label = QtWidgets.QLabel(self.available_dlc_group)
        self.available_dlc_label.setObjectName("available_dlc_label")
        self.available_dlc_group_layout.addWidget(self.available_dlc_label)
        self.available_dlc_scroll = QtWidgets.QScrollArea(self.available_dlc_group)
        self.available_dlc_scroll.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.available_dlc_scroll.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.available_dlc_scroll.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.available_dlc_scroll.setWidgetResizable(True)
        self.available_dlc_scroll.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.available_dlc_scroll.setObjectName("available_dlc_scroll")
        self.available_dlc_contents = QtWidgets.QWidget()
        self.available_dlc_contents.setGeometry(QtCore.QRect(0, 0, 255, 16))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.available_dlc_contents.sizePolicy().hasHeightForWidth())
        self.available_dlc_contents.setSizePolicy(sizePolicy)
        self.available_dlc_contents.setObjectName("available_dlc_contents")
        self.available_dlc_contents_layout = QtWidgets.QVBoxLayout(self.available_dlc_contents)
        self.available_dlc_contents_layout.setContentsMargins(0, 0, 9, 0)
        self.available_dlc_contents_layout.setObjectName("available_dlc_contents_layout")
        self.available_dlc_scroll.setWidget(self.available_dlc_contents)
        self.available_dlc_group_layout.addWidget(self.available_dlc_scroll)
        self.game_dlc_layout.addWidget(self.available_dlc_group)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.game_dlc_layout.addItem(spacerItem)

        self.retranslateUi(GameDlc)
        QtCore.QMetaObject.connectSlotsByName(GameDlc)

    def retranslateUi(self, GameDlc):
        _translate = QtCore.QCoreApplication.translate
        GameDlc.setWindowTitle(_translate("GameDlc", "GameDlc"))
        self.installed_dlc_group.setTitle(_translate("GameDlc", "Installed DLCs"))
        self.installed_dlc_label.setText(_translate("GameDlc", "No Downloadable Content has been installed."))
        self.available_dlc_group.setTitle(_translate("GameDlc", "Available DLCs"))
        self.available_dlc_label.setText(_translate("GameDlc", "No Downloadable Content is available"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    GameDlc = QtWidgets.QWidget()
    ui = Ui_GameDlc()
    ui.setupUi(GameDlc)
    GameDlc.show()
    sys.exit(app.exec_())



import os
import platform
from logging import getLogger
from typing import Tuple, Iterable, List

from PyQt5.QtCore import Qt, QThreadPool, QRunnable, pyqtSlot, pyqtSignal
from PyQt5.QtWidgets import QGroupBox, QListWidgetItem, QFileDialog, QMessageBox

from rare.lgndr.api_exception import LgndrException
from rare.shared import LegendaryCoreSingleton, GlobalSignalsSingleton
from rare.ui.components.tabs.games.import_sync.egl_sync_group import Ui_EGLSyncGroup
from rare.ui.components.tabs.games.import_sync.egl_sync_list_group import (
    Ui_EGLSyncListGroup,
)
from rare.utils.extra_widgets import PathEdit
from rare.utils.models import PathSpec
from rare.utils.misc import WineResolver

logger = getLogger("EGLSync")


class EGLSyncGroup(QGroupBox, Ui_EGLSyncGroup):
    def __init__(self, parent=None):
        super(EGLSyncGroup, self).__init__(parent=parent)
        self.setupUi(self)
        self.core = LegendaryCoreSingleton()
        self.egl_path_info.setProperty("infoLabel", 1)

        self.thread_pool = QThreadPool.globalInstance()

        if platform.system() == "Windows":
            self.egl_path_edit_label.setVisible(False)
            self.egl_path_info_label.setVisible(False)
            self.egl_path_info.setVisible(False)
        else:
            self.egl_path_edit = PathEdit(
                path=self.core.egl.programdata_path,
                placeholder=self.tr(
                    "Path to the Wine prefix where EGL is installed, or the Manifests folder"
                ),
                file_type=QFileDialog.DirectoryOnly,
                edit_func=self.egl_path_edit_edit_cb,
                save_func=self.egl_path_edit_save_cb,
                parent=self,
            )
            self.egl_path_edit.textChanged.connect(self.egl_path_changed)
            self.egl_path_edit_layout.addWidget(self.egl_path_edit)

            if not self.core.egl.programdata_path:
                self.egl_path_info.setText(self.tr("Updating..."))
                wine_resolver = WineResolver(
                    PathSpec.egl_programdata, "default"
                )
                wine_resolver.signals.result_ready.connect(self.wine_resolver_cb)
                self.thread_pool.start(wine_resolver)
            else:
                self.egl_path_info_label.setVisible(False)
                self.egl_path_info.setVisible(False)

        self.egl_sync_check.setChecked(self.core.egl_sync_enabled)
        self.egl_sync_check.stateChanged.connect(self.egl_sync_changed)

        self.import_list = EGLSyncListGroup(export=False, parent=self)
        self.import_export_layout.addWidget(self.import_list)
        self.export_list = EGLSyncListGroup(export=True, parent=self)
        self.import_export_layout.addWidget(self.export_list)

        # self.egl_watcher = QFileSystemWatcher([self.egl_path_edit.text()], self)
        # self.egl_watcher.directoryChanged.connect(self.update_lists)

        self.update_lists()

    def wine_resolver_cb(self, path):
        self.egl_path_info.setText(path)
        if not path:
            self.egl_path_info.setText(
                self.tr(
                    "Default Wine prefix is unset, or path does not exist. "
                    "Create it or configure it in Settings -> Linux."
                )
            )
        elif not os.path.exists(path):
            self.egl_path_info.setText(
                self.tr(
                    "Default Wine prefix is set but EGL manifests path does not exist. "
                    "Your configured default Wine prefix might not be where EGL is installed."
                )
            )
        else:
            self.egl_path_edit.setText(path)

    @staticmethod
    def egl_path_edit_edit_cb(path) -> Tuple[bool, str, str]:
        if not path:
            return True, path, ""
        if os.path.exists(os.path.join(path, "system.reg")) and os.path.exists(
                os.path.join(path, "dosdevices/c:")
        ):
            # path is a wine prefix
            path = os.path.join(
                path,
                "dosdevices/c:",
                "ProgramData/Epic/EpicGamesLauncher/Data/Manifests",
            )
        elif not path.rstrip("/").endswith(
                "ProgramData/Epic/EpicGamesLauncher/Data/Manifests"
        ):
            # lower() might or might not be needed in the check
            return False, path, PathEdit.reasons.wrong_path
        if os.path.exists(path):
            return True, path, ""
        return False, path, PathEdit.reasons.dir_not_exist

    def egl_path_edit_save_cb(self, path):
        if not path or not os.path.exists(path):
            # This is the same as "--unlink"
            self.core.egl.programdata_path = None
            self.core.lgd.config.remove_option("Legendary", "egl_programdata")
            self.core.lgd.config.remove_option("Legendary", "egl_sync")
            # remove EGL GUIDs from all games, DO NOT remove .egstore folders because that would fuck things up.
            for igame in self.core.get_installed_list():
                igame.egl_guid = ""
                self.core.install_game(igame)
        else:
            self.core.egl.programdata_path = path
            self.core.lgd.config.set("Legendary", "egl_programdata", path)

        self.core.lgd.save_config()

    def egl_path_changed(self, path):
        if self.egl_path_edit.is_valid:
            self.egl_sync_check.setEnabled(bool(path))
        self.egl_sync_check.setCheckState(Qt.Unchecked)
        # self.egl_watcher.removePaths([p for p in self.egl_watcher.directories()])
        # self.egl_watcher.addPaths([path])
        self.update_lists()

    def egl_sync_changed(self, state):
        if state == Qt.Unchecked:
            self.import_list.setEnabled(bool(self.import_list.items))
            self.export_list.setEnabled(bool(self.export_list.items))
            self.core.lgd.config.remove_option("Legendary", "egl_sync")
        else:
            self.core.lgd.config.set("Legendary", "egl_sync", str(True))
            # lk: do import/export here since automatic sync was selected
            self.import_list.mark(Qt.Checked)
            self.export_list.mark(Qt.Checked)
            sync_worker = EGLSyncWorker(self.import_list, self.export_list)
            self.thread_pool.start(sync_worker)
            self.import_list.setEnabled(False)
            self.export_list.setEnabled(False)
            # self.update_lists()
        self.core.lgd.save_config()

    def update_lists(self):
        # self.egl_watcher.blockSignals(True)
        if have_path := bool(self.core.egl.programdata_path) and os.path.exists(
            self.core.egl.programdata_path
        ):
            # NOTE: need to clear known manifests to force refresh
            self.core.egl.manifests.clear()
        self.egl_sync_check_label.setEnabled(have_path)
        self.egl_sync_check.setEnabled(have_path)
        self.import_list.populate(have_path)
        self.import_list.setEnabled(have_path)
        self.export_list.populate(have_path)
        self.export_list.setEnabled(have_path)
        # self.egl_watcher.blockSignals(False)


class EGLSyncListItem(QListWidgetItem):
    def __init__(self, game, export: bool, parent=None):
        super(EGLSyncListItem, self).__init__(parent=parent)
        self.setFlags(Qt.ItemIsSelectable | Qt.ItemIsEnabled | Qt.ItemIsUserCheckable)
        self.setCheckState(Qt.Unchecked)
        self.core = LegendaryCoreSingleton()
        self.game = game
        self.export = export
        if export:
            self.setText(game.title)
        else:  # import
            self.setText(self.core.get_game(game.app_name).app_title)

    def is_checked(self) -> bool:
        return self.checkState() == Qt.Checked

    def action(self) -> str:
        error = ""
        if self.export:
            try:
                self.core.egl_export(self.game.app_name)
            except LgndrException as ret:
                error = ret.message
        else:
            try:
                self.core.egl_import(self.game.app_name)
            except LgndrException as ret:
                error = ret.message
        return error

    @property
    def app_name(self):
        return self.game.app_name

    @property
    def app_title(self):
        return self.game.app_title


class EGLSyncListGroup(QGroupBox, Ui_EGLSyncListGroup):
    action_errors = pyqtSignal(list)

    def __init__(self, export: bool, parent=None):
        super(EGLSyncListGroup, self).__init__(parent=parent)
        self.setupUi(self)
        self.core = LegendaryCoreSingleton()
        self.signals = GlobalSignalsSingleton()
        self.list.setProperty("noBorder", 1)
        # TODO: Convert the CSS and the code to adhere to NoFrame
        # self.list.setFrameShape(self.list.NoFrame)

        self.export = export

        if export:
            self.setTitle(self.tr("Exportable games"))
            self.label.setText(self.tr("No games to export to EGL"))
            self.action_button.setText(self.tr("Export"))
            self.list_func = self.core.egl_get_exportable
        else:
            self.setTitle(self.tr("Importable games"))
            self.label.setText(self.tr("No games to import from EGL"))
            self.action_button.setText(self.tr("Import"))
            self.list_func = self.core.egl_get_importable

        self.list.itemDoubleClicked.connect(
            lambda item: item.setCheckState(Qt.Unchecked)
            if item.checkState() != Qt.Unchecked
            else item.setCheckState(Qt.Checked)
        )
        self.list.itemChanged.connect(self.has_selected)

        self.select_all_button.clicked.connect(lambda: self.mark(Qt.Checked))
        self.select_none_button.clicked.connect(lambda: self.mark(Qt.Unchecked))

        self.action_button.clicked.connect(self.action)

        self.action_errors.connect(self.show_errors)

    def has_selected(self):
        for item in self.items:
            if item.is_checked():
                self.action_button.setEnabled(True)
                return
        self.action_button.setEnabled(False)

    def mark(self, state):
        for item in self.items:
            item.setCheckState(state)

    def populate(self, enabled: bool):
        if enabled:
            self.list.clear()
            for item in self.list_func():
                try:
                    i = EGLSyncListItem(item, self.export, self.list)
                except AttributeError:
                    logger.error(f"{item.app_name} does not work. Ignoring")
                else:
                    self.list.addItem(i)
        self.label.setVisible(not enabled or not bool(self.list.count()))
        self.list.setVisible(enabled and bool(self.list.count()))
        self.buttons_widget.setVisible(enabled and bool(self.list.count()))

    def action(self):
        imported: List = []
        errors: List = []
        for item in self.items:
            if item.is_checked():
                if e := item.action():
                    errors.append(e)
                else:
                    imported.append(item.app_name)
                    self.list.takeItem(self.list.row(item))
        if not self.export and imported:
            self.signals.update_gamelist.emit(imported)
        self.populate(True)
        if errors:
            self.action_errors.emit(errors)

    @pyqtSlot(list)
    def show_errors(self, errors: List):
        QMessageBox.warning(
            self.parent(),
            self.tr("The following errors occurred while {}.").format(
                self.tr("exporting") if self.export else self.tr("importing")
            ),
            "\n".join(errors),
        )

    @property
    def items(self) -> Iterable[EGLSyncListItem]:
        # for i in range(self.list.count()):
        #     yield self.list.item(i)
        return [self.list.item(i) for i in range(self.list.count())]


class EGLSyncWorker(QRunnable):
    def __init__(self, import_list: EGLSyncListGroup, export_list: EGLSyncListGroup):
        super(EGLSyncWorker, self).__init__()
        self.setAutoDelete(True)
        self.import_list = import_list
        self.export_list = export_list

    @pyqtSlot()
    def run(self):
        self.import_list.action()
        self.export_list.action()



import os
import platform
import shutil
from logging import getLogger
from pathlib import Path

from PyQt5.QtCore import (
    Qt,
    pyqtSignal,
    QThreadPool,
    pyqtSlot,
)
from PyQt5.QtWidgets import (
    QHBoxLayout,
    QMenu,
    QProgressBar,
    QPushButton,
    QWidget,
    QMessageBox,
    QWidgetAction,
)
from legendary.models.game import Game, InstalledGame

from rare.models.install import InstallOptionsModel
from rare.shared import (
    LegendaryCoreSingleton,
    GlobalSignalsSingleton,
    ArgumentsSingleton,
)
from rare.shared import ImageManagerSingleton
from rare.shared.image_manager import ImageSize
from rare.ui.components.tabs.games.game_info.game_info import Ui_GameInfo
from rare.utils.legendary_utils import VerifyWorker
from rare.utils.misc import get_size
from rare.utils.steam_grades import SteamWorker
from rare.widgets.image_widget import ImageWidget
from .move_game import CopyGameInstallation, MoveGamePopUp, is_game_dir

logger = getLogger("GameInfo")


class GameInfo(QWidget, Ui_GameInfo):
    igame: InstalledGame
    game: Game = None
    verify_threads = dict()
    verification_finished = pyqtSignal(InstalledGame)
    uninstalled = pyqtSignal(str)

    def __init__(self, parent, game_utils):
        super(GameInfo, self).__init__(parent=parent)
        self.setupUi(self)
        self.core = LegendaryCoreSingleton()
        self.signals = GlobalSignalsSingleton()
        self.args = ArgumentsSingleton()
        self.image_manager = ImageManagerSingleton()
        self.game_utils = game_utils

        self.image = ImageWidget(self)
        self.image.setFixedSize(ImageSize.Display)
        self.layout_game_info.insertWidget(0, self.image, alignment=Qt.AlignTop)

        if platform.system() == "Windows":
            self.lbl_grade.setVisible(False)
            self.grade.setVisible(False)
        else:
            self.steam_worker: SteamWorker = SteamWorker(self.core)
            self.steam_worker.signals.rating.connect(self.grade.setText)
            self.steam_worker.setAutoDelete(False)

        self.game_actions_stack.setCurrentIndex(0)
        self.install_button.setText(self.tr("Link to Origin/Launch"))
        self.game_actions_stack.resize(self.game_actions_stack.minimumSize())

        self.uninstall_button.clicked.connect(self.uninstall)
        self.verify_button.clicked.connect(self.verify)

        self.verify_pool = QThreadPool()
        self.verify_pool.setMaxThreadCount(2)
        if self.args.offline:
            self.repair_button.setDisabled(True)
        else:
            self.repair_button.clicked.connect(self.repair)

        self.install_button.clicked.connect(lambda: self.game_utils.launch_game(self.game.app_name))

        self.move_game_pop_up = MoveGamePopUp()
        self.move_action = QWidgetAction(self)
        self.move_action.setDefaultWidget(self.move_game_pop_up)
        self.move_button.setMenu(QMenu())
        self.move_button.menu().addAction(self.move_action)

        self.progress_of_moving = QProgressBar()
        self.existing_game_dir = False
        self.is_moving = False
        self.game_moving = None
        self.dest_path_with_suffix = None

        self.widget_container = QWidget()
        box_layout = QHBoxLayout()
        box_layout.setContentsMargins(0, 0, 0, 0)
        box_layout.addWidget(self.move_button)
        self.widget_container.setLayout(box_layout)
        index = self.move_stack.addWidget(self.widget_container)
        self.move_stack.setCurrentIndex(index)
        self.move_game_pop_up.move_clicked.connect(self.move_button.menu().close)
        self.move_game_pop_up.move_clicked.connect(self.move_game)
        self.move_game_pop_up.browse_done.connect(self.show_menu_after_browse)

    def uninstall(self):
        if self.game_utils.uninstall_game(self.game.app_name):
            self.game_utils.update_list.emit(self.game.app_name)
            self.uninstalled.emit(self.game.app_name)

    @pyqtSlot()
    def repair(self):
        """ This function is to be called from the button only """
        repair_file = os.path.join(self.core.lgd.get_tmp_path(), f"{self.igame.app_name}.repair")
        if not os.path.exists(repair_file):
            QMessageBox.warning(
                self,
                self.tr("Error - {}").format(self.igame.title),
                self.tr(
                    "Repair file does not exist or game does not need a repair. Please verify game first"
                ),
            )
            return
        self.repair_game(self.igame)

    def repair_game(self, igame: InstalledGame):
        game = self.core.get_game(igame.app_name)
        ans = False
        if igame.version != game.app_version(igame.platform):
            ans = QMessageBox.question(
                self,
                self.tr("Repair and update?"),
                self.tr(
                    "There is an update for <b>{}</b> from <b>{}</b> to <b>{}</b>."
                    "Do you want to update the game while repairing it?"
                ).format(igame.title, igame.version, game.app_version(igame.platform)),
            ) == QMessageBox.Yes
        self.signals.install_game.emit(
            InstallOptionsModel(
                app_name=igame.app_name, repair_mode=True, repair_and_update=ans, update=True
            )
        )

    @pyqtSlot()
    def verify(self):
        """ This function is to be called from the button only """
        if not os.path.exists(self.igame.install_path):
            logger.error(f"Installation path {self.igame.install_path} for {self.igame.title} does not exist")
            QMessageBox.warning(
                self,
                self.tr("Error - {}").format(self.igame.title),
                self.tr("Installation path for <b>{}</b> does not exist. Cannot continue.").format(self.igame.title),
            )
            return
        self.verify_game(self.igame)

    def verify_game(self, igame: InstalledGame):
        self.verify_widget.setCurrentIndex(1)
        verify_worker = VerifyWorker(igame.app_name)
        verify_worker.signals.status.connect(self.verify_status)
        verify_worker.signals.result.connect(self.verify_result)
        verify_worker.signals.error.connect(self.verify_error)
        self.verify_progress.setValue(0)
        self.verify_threads[igame.app_name] = verify_worker
        self.verify_pool.start(verify_worker)
        self.move_button.setEnabled(False)

    def verify_cleanup(self, app_name: str):
        self.verify_widget.setCurrentIndex(0)
        self.verify_threads.pop(app_name)
        self.move_button.setEnabled(True)
        self.verify_button.setEnabled(True)

    @pyqtSlot(str, str)
    def verify_error(self, app_name, message):
        self.verify_cleanup(app_name)
        igame = self.core.get_installed_game(app_name)
        QMessageBox.warning(
            self,
            self.tr("Error - {}").format(igame.title),
            message
        )

    @pyqtSlot(str, int, int, float, float)
    def verify_status(self, app_name, num, total, percentage, speed):
        # checked, max, app_name
        if app_name == self.game.app_name:
            self.verify_progress.setValue(num * 100 // total)

    @pyqtSlot(str, bool, int, int)
    def verify_result(self, app_name, success, failed, missing):
        self.verify_cleanup(app_name)
        self.repair_button.setDisabled(success)
        igame = self.core.get_installed_game(app_name)
        if success:
            QMessageBox.information(
                self,
                self.tr("Summary - {}").format(igame.title),
                self.tr("<b>{}</b> has been verified successfully. "
                        "No missing or corrupt files found").format(igame.title),
            )
            self.verification_finished.emit(igame)
        else:
            ans = QMessageBox.question(
                self,
                self.tr("Summary - {}").format(igame.title),
                self.tr(
                    "Verification failed, <b>{}</b> file(s) corrupted, <b>{}</b> file(s) are missing. "
                    "Do you want to repair them?"
                ).format(failed, missing),
                QMessageBox.Yes | QMessageBox.No,
                QMessageBox.Yes,
            )
            if ans == QMessageBox.Yes:
                self.repair_game(igame)

    @pyqtSlot(str)
    def move_game(self, dest_path):
        dest_path = Path(dest_path)
        install_path = Path(self.igame.install_path)
        self.dest_path_with_suffix = dest_path.joinpath(install_path.stem)

        if self.dest_path_with_suffix.is_dir():
            self.existing_game_dir = is_game_dir(install_path, self.dest_path_with_suffix)

        if not self.existing_game_dir:
            for i in dest_path.iterdir():
                if install_path.stem in i.stem:
                    warn_msg = QMessageBox()
                    warn_msg.setText(self.tr("Destination file/directory exists."))
                    warn_msg.setInformativeText(
                        self.tr("Do you really want to overwrite it? This will delete {}").format(
                            self.dest_path_with_suffix
                        )
                    )
                    warn_msg.addButton(QPushButton(self.tr("Yes")), QMessageBox.YesRole)
                    warn_msg.addButton(QPushButton(self.tr("No")), QMessageBox.NoRole)

                    response = warn_msg.exec()

                    if response == 0:
                        # Not using pathlib, since we can't delete not-empty folders. With shutil we can.
                        if self.dest_path_with_suffix.is_dir():
                            shutil.rmtree(self.dest_path_with_suffix)
                        else:
                            self.dest_path_with_suffix.unlink()
                    else:
                        return

        self.move_stack.addWidget(self.progress_of_moving)
        self.move_stack.setCurrentWidget(self.progress_of_moving)

        self.game_moving = self.igame.app_name
        self.is_moving = True

        self.verify_button.setEnabled(False)

        if self.move_game_pop_up.find_mount(dest_path) != self.move_game_pop_up.find_mount(install_path):
            # Destination dir on different drive
            self.start_copy_diff_drive()
        else:
            # Destination dir on same drive
            shutil.move(self.igame.install_path, dest_path)
            self.set_new_game(self.dest_path_with_suffix)

    def update_progressbar(self, progress_int):
        self.progress_of_moving.setValue(progress_int)

    def start_copy_diff_drive(self):
        copy_worker = CopyGameInstallation(
            install_path=self.igame.install_path,
            dest_path=self.dest_path_with_suffix,
            is_existing_dir=self.existing_game_dir,
            igame=self.igame,
        )

        copy_worker.signals.progress.connect(self.update_progressbar)
        copy_worker.signals.finished.connect(self.set_new_game)
        copy_worker.signals.no_space_left.connect(self.warn_no_space_left)
        QThreadPool.globalInstance().start(copy_worker)

    def move_helper_clean_up(self):
        self.move_stack.setCurrentWidget(self.move_button)
        self.move_game_pop_up.refresh_indicator()
        self.is_moving = False
        self.game_moving = None
        self.verify_button.setEnabled(True)
        self.move_button.setEnabled(True)

    # This func does the needed UI changes, e.g. changing back to the initial move tool button and other stuff
    def warn_no_space_left(self):
        err_msg = QMessageBox()
        err_msg.setText(self.tr("Out of space or unknown OS error occured."))
        err_msg.exec()
        self.move_helper_clean_up()

    # Sets all needed variables to the new path.
    def set_new_game(self, dest_path_with_suffix):
        self.install_path.setText(str(dest_path_with_suffix))
        self.igame.install_path = str(dest_path_with_suffix)
        self.core.lgd.set_installed_game(self.igame.app_name, self.igame)
        self.move_game_pop_up.install_path = self.igame.install_path

        self.move_helper_clean_up()

    # We need to re-show the menu, as after clicking on browse, the whole menu gets closed.
    # Otherwise, the user would need to click on the move button again to open it again.
    def show_menu_after_browse(self):
        self.move_button.showMenu()

    def update_game(self, app_name: str):
        self.game = self.core.get_game(app_name)
        self.igame = self.core.get_installed_game(self.game.app_name)
        self.title.setTitle(self.game.app_title)

        self.image.setPixmap(self.image_manager.get_pixmap(self.game.app_name, color=True))

        self.app_name.setText(self.game.app_name)
        if self.igame:
            self.version.setText(self.igame.version)
        else:
            self.version.setText(self.game.app_version(self.igame.platform if self.igame else "Windows"))
        self.dev.setText(self.game.metadata["developer"])

        if self.igame:
            self.install_size.setText(get_size(self.igame.install_size))
            self.install_path.setText(self.igame.install_path)
            self.install_size.setVisible(True)
            self.install_path.setVisible(True)
            self.platform.setText(self.igame.platform)
        else:
            self.install_size.setVisible(False)
            self.install_path.setVisible(False)
            self.platform.setText("Windows")

        if not self.igame:
            # origin game
            self.uninstall_button.setDisabled(True)
            self.verify_button.setDisabled(True)
            self.repair_button.setDisabled(True)
            self.game_actions_stack.setCurrentIndex(1)
        else:
            self.uninstall_button.setDisabled(False)
            self.verify_button.setDisabled(False)
            if not self.args.offline:
                self.repair_button.setDisabled(
                    not os.path.exists(os.path.join(self.core.lgd.get_tmp_path(), f"{self.igame.app_name}.repair"))
                )
            self.game_actions_stack.setCurrentIndex(0)

        try:
            is_ue = self.core.get_asset(app_name).namespace == "ue"
        except ValueError:
            is_ue = False
        grade_visible = not is_ue and platform.system() != "Windows"
        self.grade.setVisible(grade_visible)
        self.lbl_grade.setVisible(grade_visible)

        if platform.system() != "Windows" and not is_ue:
            self.grade.setText(self.tr("Loading"))
            self.steam_worker.set_app_name(self.game.app_name)
            QThreadPool.globalInstance().start(self.steam_worker)

        if len(self.verify_threads.keys()) == 0 or not self.verify_threads.get(self.game.app_name):
            self.verify_widget.setCurrentIndex(0)
        elif self.verify_threads.get(self.game.app_name):
            self.verify_widget.setCurrentIndex(1)
            self.verify_progress.setValue(
                int(
                    self.verify_threads[self.game.app_name].num
                    / self.verify_threads[self.game.app_name].total
                    * 100
                )
            )

        # If the game that is currently moving matches with the current app_name, we show the progressbar.
        # Otherwhise, we show the move tool button.
        if self.igame is not None:
            if self.game_moving == self.igame.app_name:
                index = self.move_stack.addWidget(self.progress_of_moving)
                self.move_stack.setCurrentIndex(index)
            else:
                index = self.move_stack.addWidget(self.move_button)
                self.move_stack.setCurrentIndex(index)

        # If a game is verifying or moving, disable both verify and moving buttons.
        if len(self.verify_threads):
            self.verify_button.setEnabled(False)
            self.move_button.setEnabled(False)
        if self.is_moving:
            self.move_button.setEnabled(False)
            self.verify_button.setEnabled(False)

        self.move_game_pop_up.update_game(app_name)




import difflib
import json
import os
from datetime import date

import requests
from PyQt5.QtCore import pyqtSignal, QRunnable, QObject, QCoreApplication

from rare.lgndr.core import LegendaryCore
from rare.shared import LegendaryCoreSingleton, ArgumentsSingleton
from rare.utils.paths import data_dir, cache_dir

replace_chars = ",;.:-_ "
url = "https://api.steampowered.com/ISteamApps/GetAppList/v2/"


class SteamWorker(QRunnable):
    class Signals(QObject):
        rating = pyqtSignal(str)

    app_name: str = ""

    def __init__(self, core: LegendaryCore):
        super(SteamWorker, self).__init__()
        self.signals = SteamWorker.Signals()
        self.core = core
        _tr = QCoreApplication.translate
        self.ratings = {
            "platinum": _tr("SteamWorker", "Platinum"),
            "gold": _tr("SteamWorker", "Gold"),
            "silver": _tr("SteamWorker", "Silver"),
            "bronze": _tr("SteamWorker", "Bronze"),
            "fail": _tr("SteamWorker", "Could not get grade"),
            "borked": _tr("SteamWorker", "unplayable"),
            "pending": _tr("SteamWorker", "Could not get grade"),
        }

    def set_app_name(self, app_name: str):
        self.app_name = app_name

    def run(self) -> None:
        self.signals.rating.emit(
            self.ratings.get(get_rating(self.app_name), self.ratings["fail"])
        )


__steam_ids_json = None
__grades_json = None


def get_rating(app_name: str):
    core = LegendaryCoreSingleton()
    args = ArgumentsSingleton()
    global __grades_json
    if __grades_json is None:
        if os.path.exists(p := os.path.join(data_dir(), "steam_ids.json")):
            grades = json.loads(open(p).read())
            __grades_json = grades
        else:
            grades = {}
            __grades_json = grades
    else:
        grades = __grades_json

    if not grades.get(app_name):
        if args.offline:
            return "pending"
        game = core.get_game(app_name)

        steam_id = get_steam_id(game.app_title)
        grade = get_grade(steam_id)
        grades[app_name] = {"steam_id": steam_id, "grade": grade}
        with open(os.path.join(data_dir(), "steam_ids.json"), "w") as f:
            f.write(json.dumps(grades))
            f.close()
        return grade
    else:
        return grades[app_name].get("grade")


# you should iniciate the module with the game's steam code
def get_grade(steam_code):
    if steam_code == 0:
        return "fail"
    steam_code = str(steam_code)
    url = "https://www.protondb.com/api/v1/reports/summaries/"
    res = requests.get(f"{url}{steam_code}.json")
    try:
        lista = json.loads(res.text)
    except json.decoder.JSONDecodeError:
        return "fail"

    return lista["tier"]


def load_json() -> dict:
    file = os.path.join(cache_dir(), "game_list.json")
    if not os.path.exists(file):
        response = requests.get(url)
        steam_ids = json.loads(response.text)["applist"]["apps"]
        ids = {}
        for game in steam_ids:
            ids[game["name"]] = game["appid"]

        with open(file, "w") as f:
            f.write(json.dumps(ids))
            f.close()
        return ids
    else:
        return json.loads(open(file, "r").read())


def get_steam_id(title: str):
    file = os.path.join(cache_dir(), "game_list.json")
    # workarounds for satisfactory
    title = title.replace("Early Access", "").replace("Experimental", "").strip()
    global __steam_ids_json
    if __steam_ids_json is None:
        if not os.path.exists(file):
            response = requests.get(url)
            ids = {}
            steam_ids = json.loads(response.text)["applist"]["apps"]
            for game in steam_ids:
                ids[game["name"]] = game["appid"]
            __steam_ids_json = ids

            with open(file, "w") as f:
                f.write(json.dumps(ids))
                f.close()
        else:
            ids = json.loads(open(file, "r").read())
            __steam_ids_json = ids
    else:
        ids = __steam_ids_json

    if title in ids.keys():
        steam_name = [title]

    else:
        steam_name = difflib.get_close_matches(title, ids.keys(), n=1)
    if steam_name:
        return ids[steam_name[0]]
    else:
        return 0


def check_time():  # this function check if it's time to update
    file = os.path.join(cache_dir(), "game_list.json")
    json_table = json.loads(open(file, "r").read())

    today = date.today()
    day = 0  # it controls how many days it's necessary for an update
    for i in "ymd":
        if i == "d":
            day = 7
        else:
            day = 0
        if int(today.strftime("%" + i)) > int(json_table["data"][i]) + day:
            return 1
        else:
            return 0



from PyQt5.QtCore import QSize
from PyQt5.QtWidgets import QMenu, QTabWidget, QWidget, QWidgetAction, QShortcut

from rare.shared import LegendaryCoreSingleton, GlobalSignalsSingleton, ArgumentsSingleton
from rare.components.tabs.account import AccountWidget
from rare.components.tabs.downloads import DownloadsTab
from rare.components.tabs.games import GamesTab
from rare.components.tabs.settings import SettingsTab
from rare.components.tabs.settings.debug import DebugSettings
from rare.components.tabs.shop import Shop
from rare.components.tabs.tab_utils import MainTabBar, TabButtonWidget
from rare.utils.misc import icon


class TabWidget(QTabWidget):
    def __init__(self, parent):
        super(TabWidget, self).__init__(parent=parent)
        self.core = LegendaryCoreSingleton()
        self.signals = GlobalSignalsSingleton()
        self.args = ArgumentsSingleton()
        disabled_tab = 3 if not self.args.offline else 1
        self.setTabBar(MainTabBar(disabled_tab))
        # lk: Figure out why this adds a white line at the top
        # lk: despite setting qproperty-drawBase to 0 in the stylesheet
        # self.setDocumentMode(True)
        # Generate Tabs
        self.games_tab = GamesTab()
        self.addTab(self.games_tab, self.tr("Games"))

        if not self.args.offline:
            # updates = self.games_tab.default_widget.game_list.updates
            self.downloadTab = DownloadsTab(self.games_tab.updates)
            self.addTab(
                self.downloadTab,
                "Downloads"
                + (
                    " (" + str(len(self.games_tab.updates)) + ")"
                    if len(self.games_tab.updates) != 0
                    else ""
                ),
            )
            self.store = Shop(self.core)
            self.addTab(self.store, self.tr("Store (Beta)"))

        self.settings = SettingsTab(self)
        if self.args.debug:
            self.settings.addTab(DebugSettings(), "Debug")

        # Space Tab
        self.addTab(QWidget(), "")
        self.setTabEnabled(disabled_tab, False)
        # Button
        self.account = QWidget()
        self.addTab(self.account, "")
        self.setTabEnabled(disabled_tab + 1, False)

        self.account_widget = AccountWidget()
        account_action = QWidgetAction(self)
        account_action.setDefaultWidget(self.account_widget)
        account_button = TabButtonWidget("mdi.account-circle", "Account", fallback_icon="fa.user")
        account_button.setMenu(QMenu())
        account_button.menu().addAction(account_action)
        self.tabBar().setTabButton(
            disabled_tab + 1, self.tabBar().RightSide, account_button
        )

        self.addTab(self.settings, icon("fa.gear"), "")

        self.settings.about.update_available_ready.connect(
            lambda: self.tabBar().setTabText(5, "(!)")
        )
        # Signals
        # set current index
        self.signals.set_main_tab_index.connect(self.setCurrentIndex)

        # update dl tab text
        self.signals.update_download_tab_text.connect(self.update_dl_tab_text)

        # Open game list on click on Games tab button
        self.tabBarClicked.connect(self.mouse_clicked)
        self.setIconSize(QSize(25, 25))

        # shortcuts
        QShortcut("Alt+1", self).activated.connect(lambda: self.setCurrentIndex(0))
        QShortcut("Alt+2", self).activated.connect(lambda: self.setCurrentIndex(1))
        QShortcut("Alt+3", self).activated.connect(lambda: self.setCurrentIndex(2))
        QShortcut("Alt+4", self).activated.connect(lambda: self.setCurrentIndex(5))

    def update_dl_tab_text(self):
        num_downloads = len(
            set(
                [i.options.app_name for i in self.downloadTab.dl_queue]
                + [i for i in self.downloadTab.update_widgets.keys()]
            )
        )

        if num_downloads != 0:
            self.setTabText(1, f"Downloads ({num_downloads})")
        else:
            self.setTabText(1, "Downloads")

    def mouse_clicked(self, tab_num):
        if tab_num == 0:
            self.games_tab.layout().setCurrentIndex(0)

        if not self.args.offline and tab_num == 2:
            self.store.load()

    def resizeEvent(self, event):
        self.tabBar().setMinimumWidth(self.width())
        super(TabWidget, self).resizeEvent(event)
